from collections import defaultdict

from dungeonsheets import features, spells, weapons
from dungeonsheets.classes.classes import CharClass, SubClass


# PHB
class Archfey(SubClass):
    """Your patron is a lord or lady of the fey, a creature of legend who holds
    secrets that were forgotten before the mortal races were born. This being's
    motivations are often inscrutable, and sometimes whimsical, and might
    involve a striving for greater magical power or the settling of age-old
    grudges. Beings of this sort include the Prince of Frost; the Queen of Air
    and Darkness, ruler of the Gloaming Court; Titania of the Summer Court; her
    consort Oberon, the Green Lord; Hyrsam, the Prince of Fools; and ancient
    hags

    """

    name = "The Archfey Patron"
    features_by_level = defaultdict(list)
    features_by_level[1] = [features.FeyPresence]
    features_by_level[6] = [features.MistyEscape]
    features_by_level[10] = [features.BeguilingDefenses]
    features_by_level[14] = [features.DarkDelirium]


class Fiend(SubClass):
    """You have made a pact with a fiend from the lower planes of
    existence, a being whose aims are evil, even if you strive against
    those aims. Such beings desire the corruption or destruction of
    all things, ultimately including you. Fiends powerful enough to
    forge a pact include demon lords such as Demogorgon, Orcus,
    Fraz'Urb-luu, and Baphomet; archdevils such as Asmodeus, Dispater,
    Mephistopheles, and Belial; pit fiends and balors that are
    especially mighty; and ultroloths and other lords of the yugoloths

    """

    name = "The Fiend Patron"
    features_by_level = defaultdict(list)
    features_by_level[1] = [features.DarkOnesBlessing]
    features_by_level[6] = [features.DarkOnesOwnLuck]
    features_by_level[10] = [features.FiendishResilience]
    features_by_level[14] = [features.HurlThroughHell]


class GreatOldOne(SubClass):
    """Your patron is a mysterious entity whose nature is utterly foreign
    to the fabric of reality. It might come from the Far Realm, the
    space beyond reality, or it could be one of the elder gods known
    only in legends. Its motives are incomprehensible to mortals, and
    its knowledge so immense and ancient that even the greatest
    libraries pale in comparison to the vast secrets it holds. The
    Great Old One might be unaware of your existence or entirely
    indifferent to you, but the secrets you have learned allow you to
    draw your magic from it.

    Entities of this type include Ghaunadar, called That Which Lurks;
    Tharizdun, the Chained God; Dendar, the Night Serpent; Zargon, the
    Returner; Great Cthulhu; and other unfathomable beings.

    """

    name = "The Great Old One Patron"
    features_by_level = defaultdict(list)
    features_by_level[1] = [features.AwakenedMind]
    features_by_level[6] = [features.EntropicWard]
    features_by_level[10] = [features.ThoughtShield]
    features_by_level[14] = [features.CreateThrall]


# SCAG
class Undying(SubClass):
    """Death holds no sway over your patron, who has unlocked the secrets
    of everlasting life, although such a prize -- like all power --
    comes at a price. Once mortal, the Undying has seen mortal
    lifetimes pass like the seasons, like the flicker of endless days
    and nights. It has the secrets of the ages to share, secrets of
    life and death. Beings of this sort include Vecna, Lord of the
    Hand and the Eye; the dread Iuz; the lich-queen Vol; the Undying
    Court of Aerenal; Vlaakith, lich-queen of the githyanki; and the
    deathless wizard Fistandantalus.

    In the Realms, Undying patrons include Larloch the Shadow King,
    legendary master of Warlock's Crypt, and Gilgeam, the God-King of
    Unther

    """

    name = "The Undying Patron"
    features_by_level = defaultdict(list)
    features_by_level[1] = [features.AmongTheDead]
    features_by_level[6] = [features.DefyDeath]
    features_by_level[10] = [features.UndyingNature]
    features_by_level[14] = [features.IndestructibleLife]


# XGTE
class Celestial(SubClass):
    """Your patron is a powerful being of the Upper Planes. You have bound
    yourself to an ancient empyrean, solar, kirin, unicorn, or other
    entity that resides in the planes of everlasting bliss. Your pact
    with that being allows you to experience the barest touch of the
    holy light that illuminates the multiverse.

    Being connected to such power can cause changes in your behavior
    and beliefs. You might find yourself driven to annihilate the
    undead, to defeat fiends, and to protect the innocent. At times,
    your heart might also be filled with a longing for the celestial
    realm of your patron, and a desire to wander that paradise for the
    rest of your days. But you know that your mission is among mortals
    for now, and that your pact binds you to bring light to the dark
    places of the world.

    """

    name = "The Celestial Patron"
    spells_known = spells_prepared = (spells.Light, spells.SacredFlame)
    features_by_level = defaultdict(list)
    features_by_level[1] = [features.HealingLight]
    features_by_level[6] = [features.RadiantSoul]
    features_by_level[10] = [features.CelestialResilience]
    features_by_level[14] = [features.SearingVengeance]


class Hexblade(SubClass):
    """You have made your pact with a mysterious entity from the
    Shadowfell -- a force that manifests in sentient magic weapons
    carved from the stuff of shadow. The mighty sword Blackrazor is
    the most notable of these weapons, which have been spread across
    the multiverse over the ages. The shadowy force behind these
    weapons can offer power to warlocks who form pacts with it. Many
    hexhlade warlocks create weapons that emulate those formed in the
    Shadowfell. Others forgo such arms, content to weave the dark
    magic of that plane into their spellcasting.

    Because the Raven Queen is known to have forged the first of these
    weapons, many sages speculate that she and the force are one and
    that the weapons, along with hexblade warlocks, are tools she uses
    to manipulate events on the Material Plane to her inscrutable ends

    """

    name = "Hexblade Patron"
    weapon_proficiencies = (weapons.MartialWeapon,)
    _proficiencies_text = ["martial weapons", "medium armor", "shields"]
    features_by_level = defaultdict(list)
    features_by_level[1] = [features.HexbladesCurse, features.HexWarrior]
    features_by_level[6] = [features.AccursedSpecter]
    features_by_level[10] = [features.ArmorOfHexes]
    features_by_level[14] = [features.MasterOfHexes]


class Warlock(CharClass):
    name = "Warlock"
    hit_dice_faces = 8
    subclass_select_level = 1
    saving_throw_proficiencies = ("wisdom", "charisma")
    primary_abilities = ("charisma",)
    _proficiencies_text = ("light Armor", "simple weapons")
    class_skill_choices = (
        "Arcana",
        "Deception",
        "History",
        "Intimidation",
        "Investigation",
        "Nature",
        "Religion",
    )
    weapon_proficiencies = (weapons.SimpleWeapon,)
    multiclass_weapon_proficiencies = weapon_proficiencies
    _multiclass_proficiencies_text = ("light armor", "simple weapons")
    features_by_level = defaultdict(list)
    features_by_level[2] = [features.EldritchInvocation]
    features_by_level[3] = [features.PactBoon]
    features_by_level[11] = [features.MysticArcanum]
    features_by_level[20] = [features.EldritchMaster]
    subclasses_available = (Archfey, Fiend, GreatOldOne, Undying, Celestial, Hexblade)
    spellcasting_ability = "charisma"
    spell_slots_by_level = {
        1: (2, 1, 0, 0, 0, 0, 0, 0, 0, 0),
        2: (2, 2, 0, 0, 0, 0, 0, 0, 0, 0),
        3: (2, 0, 2, 0, 0, 0, 0, 0, 0, 0),
        4: (3, 0, 2, 0, 0, 0, 0, 0, 0, 0),
        5: (3, 0, 0, 3, 0, 0, 0, 0, 0, 0),
        6: (3, 0, 0, 3, 0, 0, 0, 0, 0, 0),
        7: (3, 0, 0, 0, 2, 0, 0, 0, 0, 0),
        8: (3, 0, 0, 0, 2, 0, 0, 0, 0, 0),
        9: (3, 0, 0, 0, 0, 2, 0, 0, 0, 0),
        10: (4, 0, 0, 0, 0, 2, 0, 0, 0, 0),
        11: (4, 0, 0, 0, 0, 3, 0, 0, 0, 0),
        12: (4, 0, 0, 0, 0, 3, 0, 0, 0, 0),
        13: (4, 0, 0, 0, 0, 3, 0, 0, 0, 0),
        14: (4, 0, 0, 0, 0, 3, 0, 0, 0, 0),
        15: (4, 0, 0, 0, 0, 3, 0, 0, 0, 0),
        16: (4, 0, 0, 0, 0, 3, 0, 0, 0, 0),
        17: (4, 0, 0, 0, 0, 4, 0, 0, 0, 0),
        18: (4, 0, 0, 0, 0, 4, 0, 0, 0, 0),
        19: (4, 0, 0, 0, 0, 4, 0, 0, 0, 0),
        20: (4, 0, 0, 0, 0, 4, 0, 0, 0, 0),
    }



from collections import defaultdict

from dungeonsheets import features, weapons
from dungeonsheets.classes.classes import CharClass, SubClass


# PHB
class CollegeOfLore(SubClass):
    """Bards of the College o f Lore know something about most things, collecting
    bits of knowledge from sources as diverse as scholarly tomes and peasant
    tales. Whether singing folk ballads in taverns or elaborate compositions in
    royal courts, these bards use their gifts to hold audiences
    spellbound. When the applause dies down, the audience members might find
    themselves questioning everything they held to be true, from their faith in
    the priesthood of the local temple to their loyalty to the king.

    The loyalty of these bards lies in the pursuit of beauty and truth, not in
    fealty to a monarch or following the tenets of a deity. A noble who keeps
    such a bard as a herald or advisor knows that the bard would rather be
    honest than politic.

    The college's members gather in libraries and sometimes in actual colleges,
    complete with classrooms and dormitories, to share their lore with one
    another. They also meet at festivals or affairs of state, where they can
    expose corruption, unravel lies, and poke fun at selfimportant figures of
    authority.

    """

    name = "College of Lore"
    features_by_level = defaultdict(list)
    features_by_level[3] = [features.LoreProficiencies, features.CuttingWords]
    features_by_level[6] = [features.AdditionalMagicalSecrets]
    features_by_level[14] = [features.PeerlessSkill]


class CollegeOfValor(SubClass):
    """Bards of the College of Valor are daring skalds whose tales keep alive the
    memory of the great heroes of the past, and thereby inspire a new
    generation of heroes. These bards gather in mead halls or around great
    bonfires to sing the deeds of the mighty, both past and present. They
    travel the land to witness great events firsthand and to ensure that the
    memory of those events doesn't pass from the world. With their songs, they
    inspire others to reach the same heights of accomplishment as the heroes of
    old

    """

    name = "College of Valor"
    weapon_proficiencies = (weapons.MartialWeapon,)
    _proficencies_text = ("martial weapons", "medium armor", "shields")
    features_by_level = defaultdict(list)
    features_by_level[3] = [features.CombatInspiration]
    features_by_level[6] = [features.BardExtraAttack]
    features_by_level[14] = [features.BardBattleMagic]


# XGTE
class CollegeOfGlamour(SubClass):
    """The College of Glamour is the home Of bards who mas- tered their craft in
    the vibrant realm of the Feywild or under the tutelage Of someone who
    dwelled there. Tutored by satyrs, eladrin, and other fey, these bards
    learn to use their magic to delight and captivate others.

    The bards of this college are regarded with a mixture of awe and
    fear. Their performances are the stuff of legend. These bards are so
    eloquent that a speech or song that one of them performs can cause captors
    tO release the bard unharmed and can lull a furious dragon into
    complacency. The same magic that allows them to quell beasts can also bend
    minds. Villainous bards Of this college can leech Off a community for
    weeks, misusing their magic to turn their hosts into thralls. Heroic bards
    of this college instead use this power to gladden the downtrodden and
    undermine oppressors.

    """

    name = "College of Glamour"
    features_by_level = defaultdict(list)
    features_by_level[3] = [
        features.MantleOfInspiration,
        features.EnthrallingPerformance,
    ]
    features_by_level[6] = [features.MantleOfMajesty]
    features_by_level[14] = [features.UnbreakableMajesty]


class CollegeOfSwords(SubClass):
    """Bards of the College of Swords are called blades, and they entertain
    through daring feats of weapon prowess. Blades perform stunts such as sword
    swallowing, knife throwing and juggling, and mock combats. Though they use
    their weapons to entertain, they are also highly trained and skilled
    warriors in their own right.

    Their talent with weapons inspires many blades to lead double lives. One
    blade might use a circus troupe as cover for nefarious deeds such as
    assassination, robbery, and blackmail. Other blades strike at the wicked,
    bringingjustice to bear against the cruel and powerful. Most troupes are
    happy to accept a blade's talent for the excitement it adds to a
    performance, but few entertainers fully trust a blade in their ranks.

    Blades who abandon their lives as entertainers have often run into trouble
    that makes maintaining their secret activities impossible. A blade caught
    stealing or engaging in vigilante justice is too great a liability for most
    troupes. With their weapon skills and magic, these blades either take up
    work as enforcers for thieves' guilds or strike out on their own as
    adventurers.

    """

    name = "College of Swords"
    features_by_level = defaultdict(list)
    weapon_prociciencies = (weapons.Scimitar,)
    _proficiencies_text = ("medium armor", "scimitar")
    features_by_level[3] = [
        features.SwordsProficiency,
        features.BardFightingStyle,
        features.BladeFlourish,
    ]
    features_by_level[6] = [features.BardExtraAttack]
    features_by_level[14] = [features.MastersFlourish]


class CollegeOfWhispers(SubClass):
    """Most folk are happy to welcome a bard into their midst. Bards of the
    College of Whispers use this to their advantage. They appear to be like
    other bards, sharing news, singing songs, and telling tales to the
    audiences they gather. In truth, the College of Whispers teaches its
    students that they are wolves among sheep. These bards use their knowledge
    and magic to uncover secrets and turn them against others through extortion
    and threats.

    Many other bards hate the College of Whispers, viewing it as a parasite
    that uses a bard's reputation to acquire wealth and power. For this reason,
    members of this college rarely reveal their true nature. They typically
    claim to follow some other college, or they keep their actual calling
    secret in order to infiltrate and exploit royal courts and other settings
    of power

    """

    name = "College of Whispers"
    features_by_level = defaultdict(list)
    features_by_level[3] = [features.PsychicBlades, features.WordsOfTerror]
    features_by_level[6] = [features.MantleOfWhispers]
    features_by_level[14] = [features.ShadowLore]


class Bard(CharClass):
    name = "Bard"
    hit_dice_faces = 8
    subclass_select_level = 3
    saving_throw_proficiencies = ("dexterity", "charisma")
    primary_abilities = ("charisma",)
    _proficiencies_text = (
        "Light armor",
        "simple weapons",
        "hand crossbows",
        "longswords",
        "rapiers",
        "shortswords",
        "three musical instruments of your choice",
    )
    weapon_proficiencies = (
        weapons.HandCrossbow,
        weapons.Longsword,
        weapons.Rapier,
        weapons.Shortsword,
        weapons.SimpleWeapon,
    )
    class_skill_choices = (
        "Acrobatics",
        "Animal Handling",
        "Arcana",
        "Athletics",
        "Deception",
        "History",
        "Insight",
        "Intimidation",
        "Investigation",
        "Medicine",
        "Nature",
        "Perception",
        "Performance",
        "Persuasion",
        "Religion",
        "Sleight of Hand",
        "Stealth",
        "Survival",
    )
    multiclass_weapon_proficiencies = ()
    _multiclass_proficiencies_text = (
        "Light Armor",
        "[choose one skill]",
        "[choose one musical instrument]",
    )
    num_skill_choices = 3
    subclasses_available = (
        CollegeOfLore,
        CollegeOfValor,
        CollegeOfGlamour,
        CollegeOfSwords,
        CollegeOfWhispers,
    )
    features_by_level = defaultdict(list)
    features_by_level[1] = [features.BardicInspiration]
    features_by_level[2] = [features.SongOfRest, features.JackOfAllTrades]
    features_by_level[3] = [features.BardExpertise]
    features_by_level[5] = [features.FontOfInspiration]
    features_by_level[6] = [features.Countercharm]
    features_by_level[10] = [features.MagicalSecrets]
    features_by_level[20] = [features.SuperiorInspiration]
    spellcasting_ability = "charisma"
    spell_slots_by_level = {
        # char_lvl: (cantrips, 1st, 2nd, 3rd, ...)
        1: (2, 2, 0, 0, 0, 0, 0, 0, 0, 0),
        2: (2, 3, 0, 0, 0, 0, 0, 0, 0, 0),
        3: (2, 4, 2, 0, 0, 0, 0, 0, 0, 0),
        4: (3, 4, 3, 0, 0, 0, 0, 0, 0, 0),
        5: (3, 4, 3, 2, 0, 0, 0, 0, 0, 0),
        6: (3, 4, 3, 3, 0, 0, 0, 0, 0, 0),
        7: (3, 4, 3, 3, 1, 0, 0, 0, 0, 0),
        8: (3, 4, 3, 3, 2, 0, 0, 0, 0, 0),
        9: (4, 4, 3, 3, 3, 1, 0, 0, 0, 0),
        10: (4, 4, 3, 3, 3, 2, 0, 0, 0, 0),
        11: (4, 4, 3, 3, 3, 2, 1, 0, 0, 0),
        12: (4, 4, 3, 3, 3, 2, 1, 0, 0, 0),
        13: (4, 4, 3, 3, 3, 2, 1, 1, 0, 0),
        14: (4, 4, 3, 3, 3, 2, 1, 1, 0, 0),
        15: (4, 4, 3, 3, 3, 2, 1, 1, 1, 0),
        16: (4, 4, 3, 3, 3, 2, 1, 1, 1, 0),
        17: (4, 4, 3, 3, 3, 2, 1, 1, 1, 1),
        18: (4, 4, 3, 3, 3, 3, 1, 1, 1, 1),
        19: (4, 4, 3, 3, 3, 3, 2, 1, 1, 1),
        20: (4, 4, 3, 3, 3, 3, 2, 2, 1, 1),
    }



from dungeonsheets.spells.spells import Spell


class AbiDalzimsHorridWilting(Spell):
    """You draw the moisture from every creature in a 30-foot cube
    centered on a point you choose within range. Each creature in that
    area must make a Constitution saving throw. Constructs and undead
    aren't affected, and plants and water elementals make this saving
    throw with disadvantage. A creature takes 12d8 necrotic damage on
    a failed save, or half as much damage on a successful one.

    Nonmagical plants in the area that aren't creatures, such as trees
    and shrubs, wither and die instantly.

    """

    name = "Abi-Dalzims Horrid Wilting"
    level = 8
    casting_time = "1 action"
    casting_range = "150 feet"
    components = ("V", "S", "M")
    materials = "A bit of sponge"
    duration = "Instantaneous"
    ritual = False
    magic_school = "Necromancy"
    classes = ("Sorcerer", "Wizard")


class AbsorbElements(Spell):
    """The spell captures some of the incoming energy, lessening its
    effect on you and storing it for your next melee attack. You have
    resistance to the triggering damage type until the start of your
    next turn. Also, the first time you hit with a melee attack on
    your next turn, the target takes an extra 1d6 damage of the
    triggering type, and the spell ends.

    **At Higher Levels:** When you cast this spell using a spell slot
    of 2nd level or higher, the extra damage increases by 1d6 for each
    slot level above 1st.

    """

    name = "Absorb Elements"
    level = 1
    casting_time = (
        "1 Reaction, which you take when you take "
        "acid, cold, fire, lightning, or thunder damage"
    )
    casting_range = "Self"
    components = ("S",)
    materials = ""
    duration = "1 round"
    ritual = False
    magic_school = "Abjuration"
    classes = ("Druid", "Ranger", "Wizard", "Sorcerer")


class AcidArrow(Spell):
    """A shimmering green arrow streaks toward a target within range and
    bursts in a spray of acid. Make a ranged spell attack against the
    target. On a hit, the target takes 4d4 acid damage immediately and
    2d4 acid damage at the end of its next turn. On a miss, the arrow
    splashes the target with acid for half as much of the initial
    damage and no damage at the end of its next turn.

    At Higher Levels. When you cast this spell using a spell slot of
    3rd level or higher, the damage (both initial and later) increases
    by 1d4 for each slot level above 2nd.

    """

    level = 2
    name = "Acid Arrow"
    casting_time = "1 action"
    casting_range = "90 ft"
    components = ("V", "S", "M")
    duration = "instantaneous"
    magic_school = "Evocation"
    classes = ("Wizard",)


class AcidSplash(Spell):
    """You hurl a bubble of acid.  Choose one creature within range, or
    choose two creatures within range that are within 5 feet of each
    other. A target must succeed on a Dexterity saving throw or take
    1d6 acid damage.

    **At Higher Levels:** This spell's damage increases by 1d6 when
    you reach 5th level (2d6), 11th level (3d6), and 17th level (4d6).

    """

    name = "Acid Splash"
    level = 0
    casting_time = "1 action"
    casting_range = "60 feet"
    components = ("V", "S")
    materials = ""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Conjuration"
    classes = ("Sorcerer", "Wizard")


class AganazzarsScorcher(Spell):
    """A line of roaring flame 30 feet long and 5 feet wide emanates from
    you in a direction you choose.  Each creature in the line must
    make a Dexterity saving throw. A creature takes 3d8 fire damage on
    a failed save, or half as much damage on a successful one.

    **At Higher Levels:** When you cast this spell using a spell slot
    of 3rd level or higher, the damage increases by 1d8 for each slot
    level above 2nd.

    """

    name = "Aganazzars Scorcher"
    level = 2
    casting_time = "1 action"
    casting_range = "30 feet"
    components = ("V", "S", "M")
    materials = "A red dragon's scale"
    duration = "Instantaneous"
    ritual = False
    magic_school = "Evocation"
    classes = ("Sorcerer", "Wizard")


class Aid(Spell):
    """Your spell bolsters your allies with toughness and resolve.
    Choose up to three
    creatures within range. Each target's hit point maximum and current hit points
    increase by 5 for the duration.

    **At Higher Levels:** When you cast this spell
    using a spell slot of 3rd level or higher, a target's hit points increase by an
    additional 5 for each slot level above 2nd.
    """

    name = "Aid"
    level = 2
    casting_time = "1 action"
    casting_range = "30 feet"
    components = ("V", "S", "M")
    materials = "A tiny strip of white cloth"
    duration = "8 hours"
    ritual = False
    magic_school = "Abjuration"
    classes = ("Cleric", "Paladin")


class Alarm(Spell):
    """You set an alarm against unwanted intrusion.
    Choose a door, a window, or an
    area within range that is no larger than a 20-foot cube. Until the spell ends,
    an alarm alerts you whenever a tiny or larger creature touches or enters the
    warded area. When you cast the spell, you can designate creatures that won't set
    off the alarm. You also choose whether the alarm is mental or audible.

    A mental alarm alerts you with a ping in your mind if you are
    within 1 mile of the warded area. This ping awakens you if you are
    sleeping.  An audible alarm produces the sound of a hand bell for
    10 seconds within 60 feet.

    """

    name = "Alarm"
    level = 1
    casting_time = "1 minute"
    casting_range = "30 feet"
    components = ("V", "S", "M")
    materials = "A tiny bell and a piece of fine silver wire"
    duration = "8 hours"
    ritual = True
    magic_school = "Abjuration"
    classes = ("Ranger", "Wizard")


class AlterSelf(Spell):
    """You assume a different form.
    When you cast the spell, choose one of the
    following options, the effects of which last for the duration of the spell.
    While the spell lasts, you can end one option as an action to gain the benefits
    of a different one.

    Aquatic Adaptation. You adapt your body to an aquatic
    environment, sprouting gills, and growing webbing between your fingers. You can
    breathe underwater and gain a swimming speed equal to your walking speed.
    Change
    Appearance. You transform your appearance. You decide what you look like,
    including your height, weight, facial features, sound of your voice, hair
    length, coloration, and distinguishing characteristics, if any. You can make
    yourself appear as a member of another race, though none of your statistics
    change. You also don't appear as a creature of a different size than you, and
    your basic shape stays the same; if you're bipedal, you can't use this spell to
    become quadrupedal, for instance. At any time for the duration of the spell, you
    can use your action to change your appearance in this way again.
    Natural
    Weapons. You grow claws, fangs, spines, horns, or a different natural weapon of
    your choice. Your unarmed strikes deal 1d6 bludgeoning, piercing, or slashing
    damage, as appropriate to the natural weapon you chose, and you are proficient
    with your unarmed strikes. Finally, the natural weapon is magic and you have a
    +1 bonus to the attack and damage rolls you make using it.
    """

    name = "Alter Self"
    level = 2
    casting_time = "1 action"
    casting_range = "Self"
    components = ("V", "S")
    materials = ""
    duration = "Concentration, up to 1 hour"
    ritual = False
    magic_school = "Transmutation"
    classes = ("Sorcerer", "Wizard")


class AnimalFriendship(Spell):
    """This spell lets you convince a beast that you mean it no harm.
    Choose a beast
    that you can see within range. It must see and hear you. If the beast's
    Intelligence is 4 or higher, the spell fails. Otherwise, the beast must succeed
    on a Wisdom saving throw or be charmed by you for the spell's duration. If you
    or one of your companions harms the target, the spell ends.

    At Higher Levels:
    When you cast this spell using a spell slot of 2nd level or higher, you can
    affect one additional beast for each slot level above 1st.
    """

    name = "Animal Friendship"
    level = 1
    casting_time = "1 action"
    casting_range = "30 feet"
    components = ("V", "S", "M")
    materials = "A morsel of food"
    duration = "Instantaneous"
    ritual = False
    magic_school = "Enchantment"
    classes = ("Bard", "Druid", "Ranger")


class AnimalMessenger(Spell):
    """By means of this spell, you use an animal to deliver a message.
    Choose a Tiny
    beast you can see within range, such as a squirrel, a blue ray, or a bat. You
    specify a location, which you must have visited, and a recipient who matches a
    general description, such as a man or woman dressed in the uniform of the town
    guard or a red-haired dwarf wearing a pointed hat. You also speak a message of
    up to twenty-five words. The target beast travels for the duration of the spell
    towards the specified location, covering about 50 miles per 24 hours for a
    flying messenger or 25 miles for other animals.

    When the messenger arrives, it
    delivers your message to the creature that you described, replicating the sound
    of your voice. The messenger speaks only to a creature matching the description
    you gave. If the messenger doesn't reach its destination before the spell ends,
    the message is lost, and the beast makes it way back to where you cast this
    spell.

    **At Higher Levels:** If you cast this spell using a spell slot of 3rd level
    or higher, the duration of the spell increases by 48 hours for each slot level
    above 2nd.
    """

    name = "Animal Messenger"
    level = 2
    casting_time = "1 action"
    casting_range = "30 feet"
    components = ("V", "S", "M")
    materials = "A morsel of food"
    duration = "24 hours"
    ritual = True
    magic_school = "Enchantment"
    classes = ("Bard", "Druid", "Ranger")


class AnimalShapes(Spell):
    """Your magic turns others into beasts.
    Choose any number of willing creatures
    that you can see within range. You transform each target into the form of a
    large or smaller beast with a challenge rating of 4 or lower. On subsequent
    turns, you can use your actions to transform affected creatures into new forms.


    The transformation lasts for the duration for each target, or until the target
    drops to 0 hit points or dies. You can choose a different form for each target.
    A target's game statistics are replaced by the statistics of the chosen beast,
    though the target retains its alignment and Intelligence, Wisdom, and Charisma
    scores. The target assumes the hit points of its new form, and when it reverts
    to its normal form, it returns to the number of hit point it had before it
    transformed. If it reverts as a result of dropping to 0 hit points, any excess
    damage carries over to its normal form. As long as the excess damage doesn't
    reduce the creature's normal form to 0 hit points, it isn't knocked unconcious.
    The creature is limited in the actions it can perform by the nature of its new
    form, and it can't speak or cast spells.

    The target's gear melds into the new
    form. The target can't activate, wield, or otherwise benefit from any of its
    equipment.
    """

    name = "Animal Shapes"
    level = 8
    casting_time = "1 action"
    casting_range = "30 feet"
    components = ("V", "S")
    materials = ""
    duration = "Concentration, up to 24 hours"
    ritual = False
    magic_school = "Transmutation"
    classes = ("Druid",)


class AnimateDead(Spell):
    """This spell creates an undead servant.
    Choose a pile of bones or a corpse of a
    Medium or Small humanoid within range. Your spell imbues the target with a foul
    mimicry of life, raising it as an undead creature. The target becomes a skeleton
    if you chose bones or a zombie if you chose a corpse (the DM has the creature's
    game statistics).

    On each of your turns, you can use a bonus action to
    mentally command any creature you made with this spell if the creature is within
    60 feet of you (if you control multiple creatures, you can command any or all
    of them at the same time, issuing the same command to each one). You decide what
    action the creature will take and where it will move during its next turn, or
    you can issue a general command, such as to guard a particular chamber or
    corridor. If you issue no commands, the creature only defends itself against
    hostile creatures. Once given an order, the creature continues to follow it
    until its task is complete.

    The creature is under your control for 24 hours,
    after which it stops obeying any command you've given it. To maintain the
    control of the creature for another 24 hours, you must cast this spell on the
    creature again before the current 24-hour period ends. This use of the spell
    reasserts your control over up to four creatures you have animated with this
    spell, rather than animating a new one.

    **At Higher Levels:** When you cast this
    spell using a spell slot of 4th level or higher, you animate or reassert control
    over two additional undead creatures for each slot level above 3rd. Each of the
    creatures must come from a different corpse or pile of bones.
    """

    name = "Animate Dead"
    level = 3
    casting_time = "1 minute"
    casting_range = "10 feet"
    components = ("V", "S", "M")
    materials = "A drop of blood, a piece of flesh, and a pinch of bone dust"
    duration = "Instantaneous"
    ritual = False
    magic_school = "Necromancy"
    classes = ("Cleric", "Wizard")


class AnimateObjects(Spell):
    """Objects come to life at your command. Choose up to ten nonmagical
    objects within range that are not being worn or carried. Medium
    targets count as two objects, Large targets count as four objects,
    Huge targets count as eight objects. You can't animate any object
    larger than Huge. Each target animates and becomes a creature
    under your control until the spell ends or until reduced to 0 hit
    points.

    As a bonus action, you can mentally command any creature you made
    with this spell if the creature is within 500 feet of you (if you
    control multiple creatures, you can command any or all of them at
    the same time, issuing the same command to each one). You decide
    what action the creature will take and where it will move during
    its next turn, or you can issue a general command, such as to
    guard a particular chamber or corridor. If you issue no commands,
    the creature only defends itself against hostile creatures. Once
    given an order, the creature continues to follow it until its task
    is complete.

    **Animated Object Statistics:**

    - Tiny -- HP: 20, AC: 18, Attack: +8 to hit, 1d4 + 4 damage, Str:
      4, Dex: 18
    - Small -- HP: 25, AC: 16, Attack: +6 to hit, 1d8 + 2 damage, Str:
      6, Dex: 14
    - Medium – HP: 40, AC: 13, Attack: +5 to hit, 2d6 + 1 damage, Str:
      10, Dex: 12
    - Large – HP: 50, AC: 10, Attack: +6 to hit, 2d10 + 2 damage, Str:
      14, Dex: 10
    - Huge – HP: 80, AC: 10, Attack: +8 to hit, 2d12 + 4 damage, Str:
      18, Dex: 6

    An animated object is a construct with AC, hit points, attacks,
    Strength, and Dexterity determine by its size. Its Constitution is
    10 and its Intelligence and Wisdom are 3, and its Charisma is
    1. Its speed is 30 feet; if the objects lack legs or other
    appendages it can use for locomotion, it instead has a flying
    speed of 30 feet and can hover. If the object is securely attached
    to a surface or larger object, such as a chain bolted to a wall,
    its speed is 0.  It has blindsight with a radius of 30 feet and is
    blind beyond that distance.  When the animated object drops to 0
    hit points, it reverts to its original object form, and any
    remaining damage carries over to its original object form.

    If you command an object to attack, it can make a single melee
    attack against a creature within 5 feet of it. It makes a slam
    attack with an attack bonus and bludgeoning damage determine by
    its size. The DM might rule that a specific object inflicts
    slashing or piercing damage based on its form.

    **At Higher:** If you cast this spell using a spell slot of 6th
    level or higher, you can animate two additional objects for each
    slot level above 5th.

    """

    name = "Animate Objects"
    level = 5
    casting_time = "1 action"
    casting_range = "120 feet"
    components = ("V", "S")
    materials = ""
    duration = "Concentration, up to 1 minute"
    ritual = False
    magic_school = "Transmutation"
    classes = ("Bard", "Sorcerer", "Wizard")


class AntilifeShell(Spell):
    """A shimmering barrier extends out from you in a 10-foot radius and moves with
    you, remaining centered on you and hedging out creatures other than undead and
    constructs.
    The barrier lasts for the duration. The barrier prevents an
    affected creature from passing or reaching through. An affected creature can
    cast spells or make attacks with ranged or reach weapons through the barrier.

    If you move so that an affect creature is forced to pass through the barrier,
    the spell ends.
    """

    name = "Antilife Shell"
    level = 5
    casting_time = "1 action"
    casting_range = "Self (10-foot radius)"
    components = ("V", "S")
    materials = ""
    duration = "Concentration, up to 1 hour"
    ritual = False
    magic_school = "Abjuration"
    classes = ("Druid",)


class AntimagicField(Spell):
    """A 10-foot-radius invisible sphere of antimagic surrounds you.  This
    area is divorced from the magical energy that suffeses the
    multiverse. Within the sphere, spells can't be cast, summoned
    creatures disappear, and even magic items become mundane. Until
    the spell ends, the spere moves with you, centered on you.

    Spells and other magical effects, except those created by an
    artifact or a deity, are suppressed in the sphere and can't
    protrude into it. A slot expended to cast a suppressed spell is
    consumed. While an effect is suppressed, it doesn't function, but
    the time it spends suppressed counts against its duration.

    Targeted Effects.
      Spells and other magical effects, such as magic missle and charm
      person, that target a creature or an object in the sphere have
      no effect on that target.
    Areas of Magic.
      The area of another spell or magical effect, such as fireball,
      can't extend into the sphere. If the sphere overlaps an area of
      magic, the part of the area that is covered by the sphere is
      suppressed. For example, the flames created by a wall of fire
      are suppressed within the sphere, creating a gap in the wall if
      the overlap is large enough.
    Spells.
      Any active spell or other magical effect on a creature or an
      object in the sphere is suppressed while the creature or object
      is in it.
    Magic Items.
      The properties and powers of magic items are suppressed in the
      sphere. Forexample, a +1 longsword in the sphere functions as a
      nonmagical longsword. A magic weapon's properties and powers are
      suppressed if it is used against a target in the sphere or
      wielded by an attacker in the sphere. If a magic weapon or piece
      of magic ammunition fully leaves the sphere (For example, if you
      fire a magic arrow or throw a magic spear at a target outside
      the sphere), the magic of the item ceases to be supressed as
      soon as it exits.
    Magical Travel.
      Teleportation and planar travel fail to work in the sphere,
      whether the sphere is the destination or the departure point for
      such magical travel. A portal to another location, world, or
      plane of existence, as well as an opening to an extradimensional
      space such as that created by the rope trick spells, temporarily
      closes while in the sphere.
    Creatures and Objects.
      A creature or object summoned or created by magic temporarily
      winks out of existence in the sphere. Such a creature instantly
      reappears once the space the creature occupied is no longer
      withinthe sphere.
    Dispel Magic.
      Spells and magical effects such as dispel magic have no effect
      on the sphere. Likewise, the spheres created by different
      antimagic field spells don't nullify each other.

    """

    name = "Antimagic Field"
    level = 8
    casting_time = "1 action"
    casting_range = "Self (10-foot-radius sphere)"
    components = ("V", "SM")
    materials = "A pinch of powdered iron or iron filings"
    duration = "Concentration, up to 1 hour"
    ritual = False
    magic_school = "Abjuration"
    classes = ("Cleric", "Wizard")


class Antipathysympathy(Spell):
    """This spell attracts or repels creatures of your choice.
    You target something
    within range, either a Huge or smaller object or creature or an area that is no
    larger than a 200-foot cube. Then specify a kind of intelligent creature, such
    as red dragons, goblins, or vampires. You invest the target with an aura that
    either attracts or repels the specified creatures for the duration. Choose
    antipathy or sympathy as the aura's effect.

    Antipathy.
    The enchantment causes
    creatures of the kind you designated to feel an intense urge to leave the area
    and avoid the target. When such a creature can see the target or comes within 60
    feet of it, the creature must succeed on a Wisdom saving throw or
    become frightened. The creature remains frightened while it can see the target
    or is within 60 feet of it. While frightened by the target, the creature must
    use its movement to move to the nearest safe spot from which it can't see the
    target. If the creature moves more than 60 feet from the target and can't see
    it, the creature is no longer frightened, but the creature becomes frightened
    again if it regains sight of the target or moves within 60 feet of it.


    Sympathy.
    The enchantment causes the specified creatures to feel an intense
    urge to approach the target while within 60 feet of it or able to see it. When
    such a creature can see the target or comes within 60 feet o fit, the creature
    must succeed on a Wisdom saving throw or use its movement on each of its turns
    to enter the area or move within reach of the target. When the creature has done
    so, it can't willingly move away from the target. If the target damages or
    otherwise harms an affected creature, the affected creature can make a
    Wisdom saving throw to end the effect, as described below.

    Ending the Effect.

    If an affected creature ends its turn while not within 60 feet of the target
    or able to see it, the creature makes a Wisdom saving throw. On a successful
    save, the creature is no longer affected by the target and recognizes the
    feeling of repugnance or attraction as magical. In addition, a creature affected
    by the spell is allowed another Wisdom saving throw every 24 hours while the
    spell persists.
    A creature that successfully saves against this effect is
    immune to it for 1 minute, after which time it can be affected again.
    """

    name = "Antipathysympathy"
    level = 8
    casting_time = "1 hour"
    casting_range = "60 feet"
    components = ("V", "S", "M")
    materials = (
        "Either a lump of alum soaked in vinegar for the antipathy effect or a drop of"
        " honey for the sympathy effect"
    )
    duration = "10 days"
    ritual = False
    magic_school = "Enchantment"
    classes = ("Druid", "Wizard")


class ArcaneEye(Spell):
    """You create an invisible, magical eye within e that hovers in the air for the
    duration.
    You mentally receive visual information from the eye, which has
    normal vision and darkvision out to 30 feet. The eye can look in every
    direction.
    As an action, you can move the eye up to 30 feet in any direction.
    There is no limit to how far away from you the eye can move, but it can't enter
    another plane of existence. A solid barrier blocks the eye's movement, but the
    eye can pass through an opening as small as 1 inch in diameter.
    """

    name = "Arcane Eye"
    level = 4
    casting_time = "1 action"
    casting_range = "30 feet"
    components = ("V", "S", "M")
    materials = "A bit of bat fur"
    duration = "Concentration, up to 1 hour"
    ritual = False
    magic_school = "Divination"
    classes = ("Wizard",)


class ArcaneGate(Spell):
    """You create linked teleportation portals that remain open for
    the duration.

    Choose two points on the ground that you can see, one point within 10 feet
    of you and one point within 500 feet of you. A circular portal, 10 feet in
    diameter, opens over each point. If the portal would open in the space
    occupied by a creature, the spell fails, and the casting is lost.

    The portals are two-dimensional glowing rings filled with mist, hovering
    inches from the ground and perpendicular to it at the points you choose.
    A ring is visible only from one side (your choice), which is the side that
    functions as a portal.

    Any creature or object entering the portal exits from the other portal as if
    the two were adjacent to each other; passing through a portal from
    the nonportal side has no effect. The mist that fills each portal is opaque
    and blocks vision through it. On your turn, you can rotate the rings as
    a bonus action so that the active side faces in a different direction.
    """

    name = "Arcane Gate"
    level = 6
    casting_time = "1 action"
    casting_range = "500 feet"
    components = ("V", "S")
    materials = ""
    duration = "Concentration, up to 10 minutes"
    ritual = False
    magic_school = "Conjuration"
    classes = ("Sorcerer", "Warlock", "Wizard")


class ArcaneLock(Spell):
    """You touch a closed door, window, gate, chest, or other entryway, and
    it becomes locked for the duration. You and the creatures you
    designate when you cast this spell can open the object normally. You
    can also set a password that, when spoken within 5 feet of the object,
    suppresses this spell for 1 minute. Otherwise, it is impassable until
    it is broken or the spell is dispelled or suppressed. Casting knock
    on the object suppresses arcane lock for 10 minutes.

    While affected
    by this spell, the object is more difficult to break or force open;
    the DC to break it or pick any locks on it increases by 10.
    """

    name = "Arcane Lock"
    level = 2
    casting_time = "1 action"
    casting_range = "Touch"
    components = ("V", "S", "M")
    materials = "Gold dust worth at least 25 gp, which the spell consumes"
    duration = "Until dispelled"
    ritual = False
    magic_school = "Abjuration"
    classes = ("Wizard",)


class ArmorOfAgathys(Spell):
    """A protective magical force surrounds you, manifesting as a spectral
    frost that covers you and your gear. You gain 5 temporary hit
    points for the duration. If a creature hits you with a melee
    attack while you have these hit points, the creature takes 5 cold
    damage.

    **At Higher Levels:** When you cast this spell using a spell slot
    of 2nd level or higher, both the temporary hit points and the cold
    damage increase by 5 for each slot

    """

    name = "Armor Of Agathys"
    level = 1
    casting_time = "1 action"
    casting_range = "Self"
    components = ("V", "S", "M")
    materials = "A cup of water"
    duration = "1 hour"
    ritual = False
    magic_school = "Abjuration"
    classes = ("Warlock",)


class ArmsOfHadar(Spell):
    """You invoke the power of Hadar, the Dark Hunger.
    Tendrils of dark energy erupt
    from you and batter all creatures within 10 feet of you. Each creature in that
    area must make a Strength saving throw. On a failed save, a target takes 2d6
    necrotic damage and can't take reactions until its next turn. On a successful
    save, the creature takes half damage, but suffers no other effect.

    At Higher
    Levels: When you cast this spell using a spell slot of 2nd level or higher, the
    damage increases by 1d6 for each slot level above 1st.
    """

    name = "Arms Of Hadar"
    level = 1
    casting_time = "1 action"
    casting_range = "Self (10-foot radius)"
    components = ("V", "S")
    materials = ""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Conjuration"
    classes = ("Warlock",)


class AstralProjection(Spell):
    """You and up to eight willing creatures within range project your astral bodies
    into the Astral Plane (the spell fails and the casting is wasted if you are
    already on that plane).
    The material body you leave behind is unconscious and
    in a state of suspended animation; it doesn't need food or air and doesn't age.


    Your astral body resembles your mortal form in almost every way, replicating
    your game statistics and possessions. The principal difference is the addition
    of a silvery cord that extends from between your shoulder blades and trails
    behind you, fading to invisibility after 1 foot. This cord is your tether to
    your material body. As long as the tether remains intact, you can find your
    way home. If the cord is cut something that can happen only when an effect
    specifically states that it does your soul and body are separated, killing you
    instantly.

    Your astral form can freely travel through the Astral Plane and can
    pass through portals there leading to any other plane. If you enter a new plane
    or return to the plane you were on when casting this spell, your body
    and possessions are transported along the silver cord, allowing you to re-enter
    your body as you enter the new plane. Your astral form is a separate
    incarnation. Any damage or other effects that apply to it have no effect on
    your physical body, nor do they persist when you return to it. The spell ends
    for you and your companions when you use your action to dismiss it. When the
    spell ends, the affected creature returns to its physical body, and it awakens.


    The spell might also end early for you or one of your companions. A successful
    dispel magic spell used against an astral or physical body ends the spell for
    that creature. If a creature's original body or its astral form drops to 0 hit
    points, the spell ends for that creature. If the spell ends and the silver cord
    is intact, the cord pulls the creature's astral form back to its body, ending
    its state of suspended animation. If you are returned to your body prematurely,
    your companions remain in their astral forms and must find their own way back to
    their bodies, usually by dropping to 0 hit points.
    """

    name = "Astral Projection"
    level = 9
    casting_time = "1 hour"
    casting_range = "10 feet"
    components = ("V", "S", "M")
    materials = (
        "For each creature you affect with this spell, you must provide one jacinth"
        " worth at least 1,000 gp and one ornately carved bar of silver worth at least"
        " 100 gp, all of which the spell consumes"
    )
    duration = "Special"
    ritual = False
    magic_school = "Necromancy"
    classes = ("Cleric", "Warlock", "Wizard")


class Augury(Spell):
    """By casting gem-inlaid sticks, rolling dragon bones, laying out
    ornate cards, or employing some other divining tool, you receive
    an omen from an otherworldly entity about the results of a
    specific course of action that you plan to take within the next 30
    minutes. The DM chooses from the following possible omens:

    - Weal, for good results
    - Woe, for bad results
    - Weal and woe, for both good and bad results
    - Nothing, for results that aren't especially good or bad

    The spell doesn't take into account any possible circumstances
    that might change the outcome, such as the casting of additional
    spells or the loss or gain of a companion.  If you cast the spell
    two or more times before completing your next long rest, there is
    a cumulative 25 percent chance for each casting after the first
    that you get a random reading. The DM makes this roll in secret.

    """

    name = "Augury"
    level = 2
    casting_time = "1 minute"
    casting_range = "Self"
    components = ("V", "S", "M")
    materials = (
        """Specially marked sticks, bones, or similar tokens worth at least 25 gp"""
    )
    duration = "Instantaneous"
    ritual = True
    magic_school = "Divination"
    classes = ("Cleric",)


class AuraOfLife(Spell):
    """Life-preserving energy radiates from you in an aura with a 30-foot radius.

    Until the spll ends, the aura moves with you, centered on you. Each nonhostile
    creature in the aura (including you) has resistance to necrotic damage, and its
    hit point maximum can't be reduced. In addition, a nonhostile, living creature
    regains 1 hit point when it starts its turn in the arua with 0 hit points.
    """

    name = "Aura Of Life"
    level = 4
    casting_time = "1 action"
    casting_range = "Self (30-foot radius)"
    components = ("V",)
    materials = ""
    duration = "Concentration, up to 10 minutes"
    ritual = False
    magic_school = "Abjuration"
    classes = ("Paladin",)


class AuraOfPurity(Spell):
    """Purifying energy radiates from you in an aura with a 30-foot radius.
    Until the
    spell ends, the aura moves with you, centered on you. Each nonhostile creature
    in the aura (including you) can't become diseased, has resistance to poison
    damage, and has advantage on saving throws against effects that cause any of the
    following conditions: blnded, charmed, deafended, frightened, paralyzed,
    poisoned, and stunned.
    """

    name = "Aura Of Purity"
    level = 4
    casting_time = "1 action"
    casting_range = "Self (30-foot radius)"
    components = ("V",)
    materials = ""
    duration = "Concentration, up to 10 minutes"
    ritual = False
    magic_school = "Abjuration"
    classes = ("Paladin",)


class AuraOfVitality(Spell):
    """Healing energy radiates from you in an aura with a 30-foot radius.
    Until the
    spell ends, the aura moves with you, centered on you. You can use a bonus action
    to cause one creature in the aura (including you) to regain 2d6 hit points.
    """

    name = "Aura Of Vitality"
    level = 3
    casting_time = "1 action"
    casting_range = "Self (30-foot radius)"
    components = ("V",)
    materials = ""
    duration = "Concentration, up to 1 minute"
    ritual = False
    magic_school = "Evocation"
    classes = ("Paladin",)


class Awaken(Spell):
    """After spending the casting time tracing magical pathways within a precious
    gemstone, you touch a huge or smaller beast or plant.
    The target must have
    either no Intelligence score or an Intelligence of 3 or less. The target gains
    an Intelligence of 10. The target also gains the ability to speak one language
    you know. If the target is a plant, it gains the ability to move its limbs,
    roots, vinces, creepers, and so forth, and it gains senses similar to a huamn's.
    Your DM chooses statistics appropriate for the awakened plant, such as the
    statistics for the awakened shrub or the awakened tree.

    The awakened beast or
    plant is charmed by you for 30 days or until you and your companions do anything
    harmful to it. When the charmed condition ends, the awakened creature chooses
    whether to remain friendly to you, based on how you treated it while it was
    charmed.
    """

    name = "Awaken"
    level = 5
    casting_time = "8 hours"
    casting_range = "Touch"
    components = ("V", "S", "M")
    materials = "An agate worth at least 1,000 gp, which the spell consumes"
    duration = "Instantaneous"
    ritual = False
    magic_school = "Transmutation"
    classes = ("Bard", "Druid")



"""
A collection of monsters from D&D 5e's SRD

This file was autogenerated by https://github.com/stravajiaxen/5e-srd-to-py
"""

from dungeonsheets.monsters.monsters import Monster
from dungeonsheets.stats import Ability


class HalfRedDragonVeteran(Monster):
    """
    Multiattack.
      The veteran makes two longsword attacks. If it has a shortsword drawn,
      it can also make a shortsword attack.
    Longsword.
      Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 7 (1d8 + 3)
      slashing damage, or 8 (1d10 + 3) slashing damage if used with two
      hands.
    Shortsword.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 6 (1d6 +
      3) piercing damage.
    Heavy Crossbow.
      Ranged Weapon Attack: +3 to hit, range 100/400 ft., one target. Hit: 6
      (1d10 + 1) piercing damage.
    Fire Breath.
      The veteran exhales fire in a 15-foot cone. Each creature in that area
      must make a DC 15 Dexterity saving throw, taking 24 (7d6) fire damage
      on a failed save, or half as much damage on a successful one.
    """
    name = "Half-Red Dragon Veteran"
    description = "Medium humanoid, any alignment"
    challenge_rating = 5
    armor_class = 18
    skills = ""
    senses = "Blindsight 10 ft., Darkvision 60 ft., Passive Perception 12"
    languages = "Common, Draconic"
    strength = Ability(16)
    dexterity = Ability(13)
    constitution = Ability(14)
    intelligence = Ability(10)
    wisdom = Ability(11)
    charisma = Ability(10)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 65
    hit_dice = "10d8"
    spells = []


class Harpy(Monster):
    """
    Multiattack.
      The harpy makes two attacks: one with its claws and one with its club.
    Claws.
      Melee Weapon Attack: +3 to hit, reach 5 ft., one target. Hit: 6 (2d4 +
      1) slashing damage.
    Club.
      Melee Weapon Attack: +3 to hit, reach 5 ft., one target. Hit: 3 (1d4 +
      1) bludgeoning damage.
    Luring Song.
      The harpy sings a magical melody. Every humanoid and giant within 300
      ft. of the harpy that can hear the song must succeed on a DC 11 Wisdom
      saving throw or be charmed until the song ends. The harpy must take a
      bonus action on its subsequent turns to continue singing. It can stop
      singing at any time. The song ends if the harpy is incapacitated.
      
      While charmed by the harpy, a target is incapacitated and ignores the
      songs of other harpies. If the charmed target is more than 5 ft. away
      from the harpy, the must move on its turn toward the harpy by the most
      direct route. It doesn't avoid opportunity attacks, but before moving
      into damaging terrain, such as lava or a pit, and whenever it takes
      damage from a source other than the harpy, a target can repeat the
      saving throw. A creature can also repeat the saving throw at the end
      of each of its turns. If a creature's saving throw is successful, the
      effect ends on it.
      
      A target that successfully saves is immune to this harpy's song for
      the next 24 hours.
    """
    name = "Harpy"
    description = "Medium monstrosity, chaotic evil"
    challenge_rating = 1
    armor_class = 11
    skills = ""
    senses = "Passive Perception 10"
    languages = "Common"
    strength = Ability(12)
    dexterity = Ability(13)
    constitution = Ability(12)
    intelligence = Ability(7)
    wisdom = Ability(10)
    charisma = Ability(13)
    speed = 20
    swim_speed = 0
    fly_speed = 40
    climb_speed = 0
    hp_max = 38
    hit_dice = "7d8"
    spells = []


class Hawk(Monster):
    """
    Keen Sight.
      The hawk has advantage on Wisdom (Perception) checks that rely on
      sight.
    Talons.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 1
      slashing damage.
    """
    name = "Hawk"
    description = "Tiny beast, unaligned"
    challenge_rating = 0
    armor_class = 13
    skills = "Perception +4"
    senses = "Passive Perception 14"
    languages = ""
    strength = Ability(5)
    dexterity = Ability(16)
    constitution = Ability(8)
    intelligence = Ability(2)
    wisdom = Ability(14)
    charisma = Ability(6)
    speed = 10
    swim_speed = 0
    fly_speed = 60
    climb_speed = 0
    hp_max = 1
    hit_dice = "1d4"
    spells = []


class HellHound(Monster):
    """
    Keen Hearing and Smell.
      The hound has advantage on Wisdom (Perception) checks that rely on
      hearing or smell.
    Pack Tactics.
      The hound has advantage on an attack roll against a creature if at
      least one of the hound's allies is within 5 ft. of the creature and
      the ally isn't incapacitated.
    Bite.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 7 (1d8 +
      3) piercing damage plus 7 (2d6) fire damage.
    Fire Breath.
      The hound exhales fire in a 15-foot cone. Each creature in that area
      must make a DC 12 Dexterity saving throw, taking 21 (6d6) fire damage
      on a failed save, or half as much damage on a successful one.
    """
    name = "Hell Hound"
    description = "Medium fiend, lawful evil"
    challenge_rating = 3
    armor_class = 15
    skills = "Perception +5"
    senses = "Darkvision 60 ft., Passive Perception 15"
    languages = "understands Infernal but can't speak it"
    strength = Ability(17)
    dexterity = Ability(12)
    constitution = Ability(14)
    intelligence = Ability(6)
    wisdom = Ability(13)
    charisma = Ability(6)
    speed = 50
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 45
    hit_dice = "7d8"
    spells = []


class Hezrou(Monster):
    """
    Magic Resistance.
      The hezrou has advantage on saving throws against spells and other
      magical effects.
    Stench.
      Any creature that starts its turn within 10 feet of the hezrou must
      succeed on a DC 14 Constitution saving throw or be poisoned until the
      start of its next turn. On a successful saving throw, the creature is
      immune to the hezrou's stench for 24 hours.
    Multiattack.
      The hezrou makes three attacks: one with its bite and two with its
      claws.
    Bite.
      Melee Weapon Attack: +7 to hit, reach 5 ft., one target. Hit: 15 (2d10
      + 4) piercing damage.
    Claws.
      Melee Weapon Attack: +7 to hit, reach 5 ft., one target. Hit: 11 (2d6
      + 4) slashing damage.
    Variant: Summon Demon.
      The demon chooses what to summon and attempts a magical summoning.
      
      A hezrou has a 30 percent chance of summoning 2d6 dretches or one
      hezrou.
      
      A summoned demon appears in an unoccupied space within 60 feet of its
      summoner, acts as an ally of its summoner, and can't summon other
      demons. It remains for 1 minute, until it or its summoner dies, or
      until its summoner dismisses it as an action.
    """
    name = "Hezrou"
    description = "Large fiend, chaotic evil"
    challenge_rating = 8
    armor_class = 16
    skills = ""
    senses = "Darkvision 120 ft., Passive Perception 11"
    languages = "Abyssal, telepathy 120 ft."
    strength = Ability(19)
    dexterity = Ability(17)
    constitution = Ability(20)
    intelligence = Ability(5)
    wisdom = Ability(12)
    charisma = Ability(13)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 136
    hit_dice = "13d10"
    spells = []


class HillGiant(Monster):
    """
    Multiattack.
      The giant makes two greatclub attacks.
    Greatclub.
      Melee Weapon Attack: +8 to hit, reach 10 ft., one target. Hit: 18 (3d8
      + 5) bludgeoning damage.
    Rock.
      Ranged Weapon Attack: +8 to hit, range 60/240 ft., one target. Hit: 21
      (3d10 + 5) bludgeoning damage.
    """
    name = "Hill Giant"
    description = "Huge giant, chaotic evil"
    challenge_rating = 5
    armor_class = 13
    skills = "Perception +2"
    senses = "Passive Perception 12"
    languages = "Giant"
    strength = Ability(21)
    dexterity = Ability(8)
    constitution = Ability(19)
    intelligence = Ability(5)
    wisdom = Ability(9)
    charisma = Ability(6)
    speed = 40
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 105
    hit_dice = "10d12"
    spells = []


class Hippogriff(Monster):
    """
    Keen Sight.
      The hippogriff has advantage on Wisdom (Perception) checks that rely
      on sight.
    Multiattack.
      The hippogriff makes two attacks: one with its beak and one with its
      claws.
    Beak.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 8 (1d10
      + 3) piercing damage.
    Claws.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 10 (2d6
      + 3) slashing damage.
    """
    name = "Hippogriff"
    description = "Large monstrosity, unaligned"
    challenge_rating = 1
    armor_class = 11
    skills = "Perception +5"
    senses = "Passive Perception 15"
    languages = ""
    strength = Ability(17)
    dexterity = Ability(13)
    constitution = Ability(13)
    intelligence = Ability(2)
    wisdom = Ability(12)
    charisma = Ability(8)
    speed = 40
    swim_speed = 0
    fly_speed = 60
    climb_speed = 0
    hp_max = 19
    hit_dice = "3d10"
    spells = []


class Hobgoblin(Monster):
    """
    Martial Advantage.
      Once per turn, the hobgoblin can deal an extra 7 (2d6) damage to a
      creature it hits with a weapon attack if that creature is within 5 ft.
      of an ally of the hobgoblin that isn't incapacitated.
    Longsword.
      Melee Weapon Attack: +3 to hit, reach 5 ft., one target. Hit: 5 (1d8 +
      1) slashing damage, or 6 (1d10 + 1) slashing damage if used with two
      hands.
    Longbow.
      Ranged Weapon Attack: +3 to hit, range 150/600 ft., one target. Hit: 5
      (1d8 + 1) piercing damage.
    """
    name = "Hobgoblin"
    description = "Medium humanoid, lawful evil"
    challenge_rating = 0.5
    armor_class = 18
    skills = ""
    senses = "Darkvision 60 ft., Passive Perception 10"
    languages = "Common, Goblin"
    strength = Ability(13)
    dexterity = Ability(12)
    constitution = Ability(12)
    intelligence = Ability(10)
    wisdom = Ability(10)
    charisma = Ability(9)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 11
    hit_dice = "2d8"
    spells = []


class Homunculus(Monster):
    """
    Telepathic Bond.
      While the homunculus is on the same plane of existence as its master,
      it can magically convey what it senses to its master, and the two can
      communicate telepathically.
    Bite.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one creature. Hit: 1
      piercing damage, and the target must succeed on a DC 10 Constitution
      saving throw or be poisoned for 1 minute. If the saving throw fails by
      5 or more, the target is instead poisoned for 5 (1d10) minutes and
      unconscious while poisoned in this way.
    """
    name = "Homunculus"
    description = "Tiny construct, neutral"
    challenge_rating = 0
    armor_class = 13
    skills = ""
    senses = "Darkvision 60 ft., Passive Perception 10"
    languages = "understands the languages of its creator but can't speak"
    strength = Ability(4)
    dexterity = Ability(15)
    constitution = Ability(11)
    intelligence = Ability(10)
    wisdom = Ability(10)
    charisma = Ability(7)
    speed = 20
    swim_speed = 0
    fly_speed = 40
    climb_speed = 0
    hp_max = 5
    hit_dice = "2d4"
    spells = []


class HornedDevil(Monster):
    """
    Devil's Sight.
      Magical darkness doesn't impede the devil's darkvision.
    Magic Resistance.
      The devil has advantage on saving throws against spells and other
      magical effects.
    Multiattack.
      The devil makes three melee attacks: two with its fork and one with
      its tail. It can use Hurl Flame in place of any melee attack.
    Fork.
      Melee Weapon Attack: +10 to hit, reach 10 ft., one target. Hit: 15
      (2d8 + 6) piercing damage.
    Tail.
      Melee Weapon Attack: +10 to hit, reach 10 ft., one target. Hit: 10
      (1d8 + 6) piercing damage. If the target is a creature other than an
      undead or a construct, it must succeed on a DC 17 Constitution saving
      throw or lose 10 (3d6) hit points at the start of each of its turns
      due to an infernal wound. Each time the devil hits the wounded target
      with this attack, the damage dealt by the wound increases by 10 (3d6).
      Any creature can take an action to stanch the wound with a successful
      DC 12 Wisdom (Medicine) check. The wound also closes if the target
      receives magical healing.
    Hurl Flame.
      Ranged Spell Attack: +7 to hit, range 150 ft., one target. Hit: 14
      (4d6) fire damage. If the target is a flammable object that isn't
      being worn or carried, it also catches fire.
    """
    name = "Horned Devil"
    description = "Large fiend, lawful evil"
    challenge_rating = 11
    armor_class = 18
    skills = ""
    senses = "Darkvision 120 ft., Passive Perception 13"
    languages = "Infernal, telepathy 120 ft."
    strength = Ability(22)
    dexterity = Ability(17)
    constitution = Ability(21)
    intelligence = Ability(12)
    wisdom = Ability(16)
    charisma = Ability(17)
    speed = 20
    swim_speed = 0
    fly_speed = 60
    climb_speed = 0
    hp_max = 178
    hit_dice = "17d10"
    spells = []


class HunterShark(Monster):
    """
    Blood Frenzy.
      The shark has advantage on melee attack rolls against any creature
      that doesn't have all its hit points.
    Water Breathing.
      The shark can breathe only underwater.
    Bite.
      Melee Weapon Attack: +6 to hit, reach 5 ft., one target. Hit: 13 (2d8
      + 4) piercing damage.
    """
    name = "Hunter Shark"
    description = "Large beast, unaligned"
    challenge_rating = 2
    armor_class = 12
    skills = "Perception +2"
    senses = "Darkvision 30 ft., Passive Perception 12"
    languages = ""
    strength = Ability(18)
    dexterity = Ability(13)
    constitution = Ability(15)
    intelligence = Ability(1)
    wisdom = Ability(10)
    charisma = Ability(4)
    speed = 0
    swim_speed = 40
    fly_speed = 0
    climb_speed = 0
    hp_max = 45
    hit_dice = "6d10"
    spells = []


class Hydra(Monster):
    """
    Hold Breath.
      The hydra can hold its breath for 1 hour.
    Multiple Heads.
      The hydra has five heads. While it has more than one head, the hydra
      has advantage on saving throws against being blinded, charmed,
      deafened, frightened, stunned, and knocked unconscious.
      
      Whenever the hydra takes 25 or more damage in a single turn, one of
      its heads dies. If all its heads die, the hydra dies.
      
      At the end of its turn, it grows two heads for each of its heads that
      died since its last turn, unless it has taken fire damage since its
      last turn. The hydra regains 10 hit points for each head regrown in
      this way.
    Reactive Heads.
      For each head the hydra has beyond one, it gets an extra reaction that
      can be used only for opportunity attacks.
    Wakeful.
      While the hydra sleeps, at least one of its heads is awake.
    Multiattack.
      The hydra makes as many bite attacks as it has heads.
    Bite.
      Melee Weapon Attack: +8 to hit, reach 10 ft., one target. Hit: 10
      (1d10 + 5) piercing damage.
    """
    name = "Hydra"
    description = "Huge monstrosity, unaligned"
    challenge_rating = 8
    armor_class = 15
    skills = "Perception +6"
    senses = "Darkvision 60 ft., Passive Perception 16"
    languages = ""
    strength = Ability(20)
    dexterity = Ability(12)
    constitution = Ability(20)
    intelligence = Ability(2)
    wisdom = Ability(10)
    charisma = Ability(7)
    speed = 30
    swim_speed = 30
    fly_speed = 0
    climb_speed = 0
    hp_max = 172
    hit_dice = "15d12"
    spells = []


class Hyena(Monster):
    """
    Pack Tactics.
      The hyena has advantage on an attack roll against a creature if at
      least one of the hyena's allies is within 5 ft. of the creature and
      the ally isn't incapacitated.
    Bite.
      Weapon Attack: +2 to hit, reach 5 ft., one target. Hit: 3 (1d6)
      piercing damage.
    """
    name = "Hyena"
    description = "Medium beast, unaligned"
    challenge_rating = 0
    armor_class = 11
    skills = "Perception +3"
    senses = "Passive Perception 13"
    languages = ""
    strength = Ability(11)
    dexterity = Ability(13)
    constitution = Ability(12)
    intelligence = Ability(2)
    wisdom = Ability(12)
    charisma = Ability(5)
    speed = 50
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 5
    hit_dice = "1d8"
    spells = []



"""
A collection of monsters from D&D 5e's SRD

This file was autogenerated by https://github.com/stravajiaxen/5e-srd-to-py
"""

from dungeonsheets.monsters.monsters import Monster
from dungeonsheets.stats import Ability


class FireElemental(Monster):
    """
    Fire Form.
      The elemental can move through a space as narrow as 1 inch wide
      without squeezing. A creature that touches the elemental or hits it
      with a melee attack while within 5 ft. of it takes 5 (1d10) fire
      damage. In addition, the elemental can enter a hostile creature's
      space and stop there. The first time it enters a creature's space on a
      turn, that creature takes 5 (1d10) fire damage and catches fire; until
      someone takes an action to douse the fire, the creature takes 5 (1d10)
      fire damage at the start of each of its turns.
    Illumination.
      The elemental sheds bright light in a 30-foot radius and dim light in
      an additional 30 ft..
    Water Susceptibility.
      For every 5 ft. the elemental moves in water, or for every gallon of
      water splashed on it, it takes 1 cold damage.
    Multiattack.
      The elemental makes two touch attacks.
    Touch.
      Melee Weapon Attack: +6 to hit, reach 5 ft., one target. Hit: 10 (2d6
      + 3) fire damage. If the target is a creature or a flammable object,
      it ignites. Until a creature takes an action to douse the fire, the
      target takes 5 (1d10) fire damage at the start of each of its turns.
    """
    name = "Fire Elemental"
    description = "Large elemental, neutral"
    challenge_rating = 5
    armor_class = 13
    skills = ""
    senses = "Darkvision 60 ft., Passive Perception 10"
    languages = "Ignan"
    strength = Ability(10)
    dexterity = Ability(17)
    constitution = Ability(16)
    intelligence = Ability(6)
    wisdom = Ability(10)
    charisma = Ability(7)
    speed = 50
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 102
    hit_dice = "12d10"
    spells = []


class FireGiant(Monster):
    """
    Multiattack.
      The giant makes two greatsword attacks.
    Greatsword.
      Melee Weapon Attack: +11 to hit, reach 10 ft., one target. Hit: 28
      (6d6 + 7) slashing damage.
    Rock.
      Ranged Weapon Attack: +11 to hit, range 60/240 ft., one target. Hit:
      29 (4d10 + 7) bludgeoning damage.
    """
    name = "Fire Giant"
    description = "Huge giant, lawful evil"
    challenge_rating = 9
    armor_class = 18
    skills = "Athletics +11, Perception +6"
    senses = "Passive Perception 16"
    languages = "Giant"
    strength = Ability(25)
    dexterity = Ability(9)
    constitution = Ability(23)
    intelligence = Ability(10)
    wisdom = Ability(14)
    charisma = Ability(13)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 162
    hit_dice = "13d12"
    spells = []


class FleshGolem(Monster):
    """
    Berserk.
      Whenever the golem starts its turn with 40 hit points or fewer, roll a
      d6. On a 6, the golem goes berserk. On each of its turns while
      berserk, the golem attacks the nearest creature it can see. If no
      creature is near enough to move to and attack, the golem attacks an
      object, with preference for an object smaller than itself. Once the
      golem goes berserk, it continues to do so until it is destroyed or
      regains all its hit points.
      
      The golem's creator, if within 60 feet of the berserk golem, can try
      to calm it by speaking firmly and persuasively. The golem must be able
      to hear its creator, who must take an action to make a DC 15 Charisma
      (Persuasion) check. If the check succeeds, the golem ceases being
      berserk. If it takes damage while still at 40 hit points or fewer, the
      golem might go berserk again.
    Aversion of Fire.
      If the golem takes fire damage, it has disadvantage on attack rolls
      and ability checks until the end of its next turn.
    Immutable Form.
      The golem is immune to any spell or effect that would alter its form.
    Lightning Absorption.
      Whenever the golem is subjected to lightning damage, it takes no
      damage and instead regains a number of hit points equal to the
      lightning damage dealt.
    Magic Resistance.
      The golem has advantage on saving throws against spells and other
      magical effects.
    Magic Weapons.
      The golem's weapon attacks are magical.
    Multiattack.
      The golem makes two slam attacks.
    Slam.
      Melee Weapon Attack: +7 to hit, reach 5 ft., one target. Hit: 13 (2d8
      + 4) bludgeoning damage.
    """
    name = "Flesh Golem"
    description = "Medium construct, neutral"
    challenge_rating = 5
    armor_class = 9
    skills = ""
    senses = "Darkvision 60 ft., Passive Perception 10"
    languages = "understands the languages of its creator but can't speak"
    strength = Ability(19)
    dexterity = Ability(9)
    constitution = Ability(18)
    intelligence = Ability(6)
    wisdom = Ability(10)
    charisma = Ability(5)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 93
    hit_dice = "11d8"
    spells = []


class FlyingSnake(Monster):
    """
    Flyby.
      The snake doesn't provoke opportunity attacks when it flies out of an
      enemy's reach.
    Bite.
      Melee Weapon Attack: +6 to hit, reach 5 ft., one target. Hit: 1
      piercing damage plus 7 (3d4) poison damage.
    """
    name = "Flying Snake"
    description = "Tiny beast, unaligned"
    challenge_rating = 0.125
    armor_class = 14
    skills = ""
    senses = "Blindsight 10 ft., Passive Perception 11"
    languages = ""
    strength = Ability(4)
    dexterity = Ability(18)
    constitution = Ability(11)
    intelligence = Ability(2)
    wisdom = Ability(12)
    charisma = Ability(5)
    speed = 30
    swim_speed = 30
    fly_speed = 60
    climb_speed = 0
    hp_max = 5
    hit_dice = "2d4"
    spells = []


class FlyingSword(Monster):
    """
    Antimagic Susceptibility.
      The sword is incapacitated while in the area of an antimagic field. If
      targeted by dispel magic, the sword must succeed on a Constitution
      saving throw against the caster's spell save DC or fall unconscious
      for 1 minute.
    False Appearance.
      While the sword remains motionless and isn't flying, it is
      indistinguishable from a normal sword.
    Longsword.
      Melee Weapon Attack: +3 to hit, reach 5 ft., one target. Hit: 5 (1d8 +
      1) slashing damage.
    """
    name = "Flying Sword"
    description = "Small construct, unaligned"
    challenge_rating = 0.25
    armor_class = 17
    skills = ""
    senses = "Blindsight 60 ft. (blind beyond this radius), Passive Perception 7"
    languages = ""
    strength = Ability(12)
    dexterity = Ability(15)
    constitution = Ability(11)
    intelligence = Ability(1)
    wisdom = Ability(5)
    charisma = Ability(1)
    speed = 0
    swim_speed = 0
    fly_speed = 50
    climb_speed = 0
    hp_max = 17
    hit_dice = "5d6"
    spells = []


class Frog(Monster):
    """
    Amphibious.
      The frog can breathe air and water
    Standing Leap.
      The frog's long jump is up to 10 ft. and its high jump is up to 5 ft.,
      with or without a running start.
    """
    name = "Frog"
    description = "Tiny beast, unaligned"
    challenge_rating = 0
    armor_class = 11
    skills = "Perception +1, Stealth +3"
    senses = "Darkvision 30 ft., Passive Perception 11"
    languages = ""
    strength = Ability(1)
    dexterity = Ability(13)
    constitution = Ability(8)
    intelligence = Ability(1)
    wisdom = Ability(8)
    charisma = Ability(3)
    speed = 20
    swim_speed = 20
    fly_speed = 0
    climb_speed = 0
    hp_max = 1
    hit_dice = "1d4"
    spells = []


class FrostGiant(Monster):
    """
    Multiattack.
      The giant makes two greataxe attacks.
    Greataxe.
      Melee Weapon Attack: +9 to hit, reach 10 ft., one target. Hit: 25
      (3d12 + 6) slashing damage.
    Rock.
      Ranged Weapon Attack: +9 to hit, range 60/240 ft., one target. Hit: 28
      (4d10 + 6) bludgeoning damage.
    """
    name = "Frost Giant"
    description = "Huge giant, neutral evil"
    challenge_rating = 8
    armor_class = 15
    skills = "Athletics +9, Perception +3"
    senses = "Passive Perception 13"
    languages = "Giant"
    strength = Ability(23)
    dexterity = Ability(9)
    constitution = Ability(21)
    intelligence = Ability(9)
    wisdom = Ability(10)
    charisma = Ability(12)
    speed = 40
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 138
    hit_dice = "12d12"
    spells = []



"""
A collection of monsters from D&D 5e's SRD

This file was autogenerated by https://github.com/stravajiaxen/5e-srd-to-py
"""

from dungeonsheets.monsters.monsters import Monster
from dungeonsheets.stats import Ability


class Unicorn(Monster):
    """
    Charge.
      If the unicorn moves at least 20 ft. straight toward a target and then
      hits it with a horn attack on the same turn, the target takes an extra
      9 (2d8) piercing damage. If the target is a creature, it must succeed
      on a DC 15 Strength saving throw or be knocked prone.
    Innate Spellcasting.
      The unicorn's innate spellcasting ability is Charisma (spell save DC
      14). The unicorn can innately cast the following spells, requiring no
      components:
      
  
      
      At will: detect evil and good, druidcraft, pass without trace
      
      1/day each: calm emotions, dispel evil and good, entangle
    Magic Resistance.
      The unicorn has advantage on saving throws against spells and other
      magical effects.
    Magic Weapons.
      The unicorn's weapon attacks are magical.
    Multiattack.
      The unicorn makes two attacks: one with its hooves and one with its
      horn.
    Hooves.
      Melee Weapon Attack: +7 to hit, reach 5 ft ., one target. Hit: 11 (2d6
      + 4) bludgeoning damage.
    Horn.
      Melee Weapon Attack: +7 to hit, reach 5 ft ., one target. Hit: 8 (1d8
      + 4) piercing damage.
    Healing Touch.
      The unicorn touches another creature with its horn. The target
      magically regains 11 (2d8 + 2) hit points. In addition, the touch
      removes all diseases and neutralizes all poisons afflicting the
      target.
    Teleport.
      The unicorn magically teleports itself and up to three willing
      creatures it can see within 5 ft. of it, along with any equipment they
      are wearing or carrying, to a location the unicorn is familiar with,
      up to 1 mile away.
    """
    name = "Unicorn"
    description = "Large celestial, lawful good"
    challenge_rating = 5
    armor_class = 12
    skills = ""
    senses = "Darkvision 60 ft., Passive Perception 13"
    languages = "Celestial, Elvish, Sylvan, telepathy 60 ft."
    strength = Ability(18)
    dexterity = Ability(14)
    constitution = Ability(15)
    intelligence = Ability(11)
    wisdom = Ability(17)
    charisma = Ability(16)
    speed = 50
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 67
    hit_dice = "9d10"
    spells = []



"""
A collection of monsters from D&D 5e's SRD

This file was autogenerated by https://github.com/stravajiaxen/5e-srd-to-py
"""

from dungeonsheets.monsters.monsters import Monster
from dungeonsheets.stats import Ability


class Camel(Monster):
    """
    Bite.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 2 (1d4)
      bludgeoning damage.
    """
    name = "Camel"
    description = "Large beast, unaligned"
    challenge_rating = 0.125
    armor_class = 9
    skills = ""
    senses = "Passive Perception 9"
    languages = ""
    strength = Ability(16)
    dexterity = Ability(8)
    constitution = Ability(14)
    intelligence = Ability(2)
    wisdom = Ability(8)
    charisma = Ability(5)
    speed = 50
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 15
    hit_dice = "2d10"
    spells = []


class Cat(Monster):
    """
    Keen Smell.
      The cat has advantage on Wisdom (Perception) checks that rely on
      smell.
    Claws.
      Melee Weapon Attack: +0 to hit, reach 5 ft., one target. Hit: 1
      slashing damage.
    """
    name = "Cat"
    description = "Tiny beast, unaligned"
    challenge_rating = 0
    armor_class = 12
    skills = "Perception +3, Stealth +4"
    senses = "Passive Perception 13"
    languages = ""
    strength = Ability(3)
    dexterity = Ability(15)
    constitution = Ability(10)
    intelligence = Ability(3)
    wisdom = Ability(12)
    charisma = Ability(7)
    speed = 40
    swim_speed = 0
    fly_speed = 0
    climb_speed = 30
    hp_max = 2
    hit_dice = "1d4"
    spells = []


class Centaur(Monster):
    """
    Charge.
      If the centaur moves at least 30 ft. straight toward a target and then
      hits it with a pike attack on the same turn, the target takes an extra
      10 (3d6) piercing damage.
    Multiattack.
      The centaur makes two attacks: one with its pike and one with its
      hooves or two with its longbow.
    Pike.
      Melee Weapon Attack: +6 to hit, reach 10 ft., one target. Hit: 9 (1d10
      + 4) piercing damage.
    Hooves.
      Melee Weapon Attack: +6 to hit, reach 5 ft., one target. Hit: 11 (2d6
      + 4) bludgeoning damage.
    Longbow.
      Ranged Weapon Attack: +4 to hit, range 150/600 ft., one target. Hit: 6
      (1d8 + 2) piercing damage.
    """
    name = "Centaur"
    description = "Large monstrosity, neutral good"
    challenge_rating = 2
    armor_class = 12
    skills = "Athletics +6, Perception +3, Survival +3"
    senses = "Passive Perception 13"
    languages = "Elvish, Sylvan"
    strength = Ability(18)
    dexterity = Ability(14)
    constitution = Ability(14)
    intelligence = Ability(9)
    wisdom = Ability(13)
    charisma = Ability(11)
    speed = 50
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 45
    hit_dice = "6d10"
    spells = []


class ChainDevil(Monster):
    """
    Devil's Sight.
      Magical darkness doesn't impede the devil's darkvision.
    Magic Resistance.
      The devil has advantage on saving throws against spells and other
      magical effects.
    Multiattack.
      The devil makes two attacks with its chains.
    Chain.
      Melee Weapon Attack: +8 to hit, reach 10 ft., one target. Hit: 11 (2d6
      + 4) slashing damage. The target is grappled (escape DC 14) if the
      devil isn't already grappling a creature. Until this grapple ends, the
      target is restrained and takes 7 (2d6) piercing damage at the start of
      each of its turns.
    Animate Chains.
      Up to four chains the devil can see within 60 feet of it magically
      sprout razor-edged barbs and animate under the devil's control,
      provided that the chains aren't being worn or carried.
      
      Each animated chain is an object with AC 20, 20 hit points, resistance
      to piercing damage, and immunity to psychic and thunder damage. When
      the devil uses Multiattack on its turn, it can use each animated chain
      to make one additional chain attack. An animated chain can grapple one
      creature of its own but can't make attacks while grappling. An
      animated chain reverts to its inanimate state if reduced to 0 hit
      points or if the devil is incapacitated or dies.
    """
    name = "Chain Devil"
    description = "Medium fiend, lawful evil"
    challenge_rating = 8
    armor_class = 16
    skills = ""
    senses = "Darkvision 120 ft., Passive Perception 11"
    languages = "Infernal, telepathy 120 ft."
    strength = Ability(18)
    dexterity = Ability(15)
    constitution = Ability(18)
    intelligence = Ability(11)
    wisdom = Ability(12)
    charisma = Ability(14)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 85
    hit_dice = "10d8"
    spells = []


class Chimera(Monster):
    """
    Multiattack.
      The chimera makes three attacks: one with its bite, one with its
      horns, and one with its claws. When its fire breath is available, it
      can use the breath in place of its bite or horns.
    Bite.
      Melee Weapon Attack: +7 to hit, reach 5 ft., one target. Hit: 11 (2d6
      + 4) piercing damage.
    Horns.
      Melee Weapon Attack: +7 to hit, reach 5 ft., one target. Hit: 10 (1d12
      + 4) bludgeoning damage.
    Claws.
      Melee Weapon Attack: +7 to hit, reach 5 ft., one target. Hit: 11 (2d6
      + 4) slashing damage.
    Fire Breath.
      The dragon head exhales fire in a 15-foot cone. Each creature in that
      area must make a DC 15 Dexterity saving throw, taking 31 (7d8) fire
      damage on a failed save, or half as much damage on a successful one.
    """
    name = "Chimera"
    description = "Large monstrosity, chaotic evil"
    challenge_rating = 6
    armor_class = 14
    skills = "Perception +8"
    senses = "Darkvision 60 ft., Passive Perception 18"
    languages = "understands Draconic but can't speak"
    strength = Ability(19)
    dexterity = Ability(11)
    constitution = Ability(19)
    intelligence = Ability(3)
    wisdom = Ability(14)
    charisma = Ability(10)
    speed = 30
    swim_speed = 0
    fly_speed = 60
    climb_speed = 0
    hp_max = 114
    hit_dice = "12d10"
    spells = []


class Chuul(Monster):
    """
    Amphibious.
      The chuul can breathe air and water.
    Sense Magic.
      The chuul senses magic within 120 feet of it at will. This trait
      otherwise works like the detect magic spell but isn't itself magical.
    Multiattack.
      The chuul makes two pincer attacks. If the chuul is grappling a
      creature, the chuul can also use its tentacles once.
    Pincer.
      Melee Weapon Attack: +6 to hit, reach 10 ft., one target. Hit: 11 (2d6
      + 4) bludgeoning damage. The target is grappled (escape DC 14) if it
      is a Large or smaller creature and the chuul doesn't have two other
      creatures grappled.
    Tentacles.
      One creature grappled by the chuul must succeed on a DC 13
      Constitution saving throw or be poisoned for 1 minute. Until this
      poison ends, the target is paralyzed. The target can repeat the saving
      throw at the end of each of its turns, ending the effect on itself on
      a success.
    """
    name = "Chuul"
    description = "Large aberration, chaotic evil"
    challenge_rating = 4
    armor_class = 16
    skills = "Perception +4"
    senses = "Darkvision 60 ft., Passive Perception 14"
    languages = "understands Deep Speech but can't speak"
    strength = Ability(19)
    dexterity = Ability(10)
    constitution = Ability(16)
    intelligence = Ability(5)
    wisdom = Ability(11)
    charisma = Ability(5)
    speed = 30
    swim_speed = 30
    fly_speed = 0
    climb_speed = 0
    hp_max = 93
    hit_dice = "11d10"
    spells = []


class ClayGolem(Monster):
    """
    Acid Absorption.
      Whenever the golem is subjected to acid damage, it takes no damage and
      instead regains a number of hit points equal to the acid damage dealt.
    Berserk.
      Whenever the golem starts its turn with 60 hit points or fewer, roll a
      d6. On a 6, the golem goes berserk. On each of its turns while
      berserk, the golem attacks the nearest creature it can see. If no
      creature is near enough to move to and attack, the golem attacks an
      object, with preference for an object smaller than itself. Once the
      golem goes berserk, it continues to do so until it is destroyed or
      regains all its hit points.
    Immutable Form.
      The golem is immune to any spell or effect that would alter its form.
    Magic Resistance.
      The golem has advantage on saving throws against spells and other
      magical effects.
    Magic Weapons.
      The golem's weapon attacks are magical.
    Multiattack.
      The golem makes two slam attacks.
    Slam.
      Melee Weapon Attack: +8 to hit, reach 5 ft., one target. Hit: 16 (2d10
      + 5) bludgeoning damage. If the target is a creature, it must succeed
      on a DC 15 Constitution saving throw or have its hit point maximum
      reduced by an amount equal to the damage taken. The target dies if
      this attack reduces its hit point maximum to 0. The reduction lasts
      until removed by the greater restoration spell or other magic.
    Haste.
      Until the end of its next turn, the golem magically gains a +2 bonus
      to its AC, has advantage on Dexterity saving throws, and can use its
      slam attack as a bonus action.
    """
    name = "Clay Golem"
    description = "Large construct, unaligned"
    challenge_rating = 9
    armor_class = 14
    skills = ""
    senses = "Darkvision 60 ft., Passive Perception 9"
    languages = "understands the languages of its creator but can't speak"
    strength = Ability(20)
    dexterity = Ability(9)
    constitution = Ability(18)
    intelligence = Ability(3)
    wisdom = Ability(8)
    charisma = Ability(1)
    speed = 20
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 133
    hit_dice = "14d10"
    spells = []


class Cloaker(Monster):
    """
    Damage Transfer.
      While attached to a creature, the cloaker takes only half the damage
      dealt to it (rounded down). and that creature takes the other half.
    False Appearance.
      While the cloaker remains motionless without its underside exposed, it
      is indistinguishable from a dark leather cloak.
    Light Sensitivity.
      While in bright light, the cloaker has disadvantage on attack rolls
      and Wisdom (Perception) checks that rely on sight.
    Multiattack.
      The cloaker makes two attacks: one with its bite and one with its
      tail.
    Bite.
      Melee Weapon Attack: +6 to hit, reach 5 ft., one creature. Hit: 10
      (2d6 + 3) piercing damage, and if the target is Large or smaller, the
      cloaker attaches to it. If the cloaker has advantage against the
      target, the cloaker attaches to the target's head, and the target is
      blinded and unable to breathe while the cloaker is attached. While
      attached, the cloaker can make this attack only against the target and
      has advantage on the attack roll. The cloaker can detach itself by
      spending 5 feet of its movement. A creature, including the target, can
      take its action to detach the cloaker by succeeding on a DC 16
      Strength check.
    Tail.
      Melee Weapon Attack: +6 to hit, reach 10 ft., one creature. Hit: 7
      (1d8 + 3) slashing damage.
    Moan.
      Each creature within 60 feet of the cloaker that can hear its moan and
      that isn't an aberration must succeed on a DC 13 Wisdom saving throw
      or become frightened until the end of the cloaker's next turn. If a
      creature's saving throw is successful, the creature is immune to the
      cloaker's moan for the next 24 hours.
    Phantasms.
      The cloaker magically creates three illusory duplicates of itself if
      it isn't in bright light. The duplicates move with it and mimic its
      actions, shifting position so as to make it impossible to track which
      cloaker is the real one. If the cloaker is ever in an area of bright
      light, the duplicates disappear.
      
      Whenever any creature targets the cloaker with an attack or a harmful
      spell while a duplicate remains, that creature rolls randomly to
      determine whether it targets the cloaker or one of the duplicates. A
      creature is unaffected by this magical effect if it can't see or if it
      relies on senses other than sight.
      
      A duplicate has the cloaker's AC and uses its saving throws. If an
      attack hits a duplicate, or if a duplicate fails a saving throw
      against an effect that deals damage, the duplicate disappears.
    """
    name = "Cloaker"
    description = "Large aberration, chaotic neutral"
    challenge_rating = 8
    armor_class = 14
    skills = "Stealth +5"
    senses = "Darkvision 60 ft., Passive Perception 11"
    languages = "Deep Speech, Undercommon"
    strength = Ability(17)
    dexterity = Ability(15)
    constitution = Ability(12)
    intelligence = Ability(13)
    wisdom = Ability(12)
    charisma = Ability(14)
    speed = 10
    swim_speed = 0
    fly_speed = 40
    climb_speed = 0
    hp_max = 78
    hit_dice = "12d10"
    spells = []


class CloudGiant(Monster):
    """
    Keen Smell.
      The giant has advantage on Wisdom (Perception) checks that rely on
      smell.
    Innate Spellcasting.
      The giant's innate spellcasting ability is Charisma. It can innately
      cast the following spells, requiring no material components:
      
  
      
      At will: detect magic, fog cloud, light
      
      3/day each: feather fall, fly, misty step, telekinesis
      
      1/day each: control weather, gaseous form
    Multiattack.
      The giant makes two morningstar attacks.
    Morningstar.
      Melee Weapon Attack: +12 to hit, reach 10 ft., one target. Hit: 21
      (3d8 + 8) piercing damage.
    Rock.
      Ranged Weapon Attack: +12 to hit, range 60/240 ft., one target. Hit:
      30 (4d10 + 8) bludgeoning damage.
    """
    name = "Cloud Giant"
    description = "Huge giant, neutral good (50%) or neutral evil (50%)"
    challenge_rating = 9
    armor_class = 14
    skills = "Insight +7, Perception +7"
    senses = "Passive Perception 17"
    languages = "Common, Giant"
    strength = Ability(27)
    dexterity = Ability(10)
    constitution = Ability(22)
    intelligence = Ability(12)
    wisdom = Ability(16)
    charisma = Ability(16)
    speed = 40
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 200
    hit_dice = "16d12"
    spells = []


class Cockatrice(Monster):
    """
    Bite.
      Melee Weapon Attack: +3 to hit, reach 5 ft., one creature. Hit: 3 (1d4
      + 1) piercing damage, and the target must succeed on a DC 11
      Constitution saving throw against being magically petrified. On a
      failed save, the creature begins to turn to stone and is restrained.
      It must repeat the saving throw at the end of its next turn. On a
      success, the effect ends. On a failure, the creature is petrified for
      24 hours.
    """
    name = "Cockatrice"
    description = "Small monstrosity, unaligned"
    challenge_rating = 0.5
    armor_class = 11
    skills = ""
    senses = "Darkvision 60 ft., Passive Perception 11"
    languages = ""
    strength = Ability(6)
    dexterity = Ability(12)
    constitution = Ability(12)
    intelligence = Ability(2)
    wisdom = Ability(13)
    charisma = Ability(5)
    speed = 20
    swim_speed = 0
    fly_speed = 40
    climb_speed = 0
    hp_max = 27
    hit_dice = "6d6"
    spells = []


class Commoner(Monster):
    """
    Club.
      Melee Weapon Attack: +2 to hit, reach 5 ft., one target. Hit: 2 (1d4)
      bludgeoning damage.
    """
    name = "Commoner"
    description = "Medium humanoid, any alignment"
    challenge_rating = 0
    armor_class = 10
    skills = ""
    senses = "Passive Perception 10"
    languages = "any one language (usually Common)"
    strength = Ability(10)
    dexterity = Ability(10)
    constitution = Ability(10)
    intelligence = Ability(10)
    wisdom = Ability(10)
    charisma = Ability(10)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 4
    hit_dice = "1d8"
    spells = []


class ConstrictorSnake(Monster):
    """
    Bite.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one creature. Hit: 5 (1d6
      + 2) piercing damage.
    Constrict.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one creature. Hit: 6 (1d8
      + 2) bludgeoning damage, and the target is grappled (escape DC 14).
      Until this grapple ends, the creature is restrained, and the snake
      can't constrict another target.
    """
    name = "Constrictor Snake"
    description = "Large beast, unaligned"
    challenge_rating = 0.25
    armor_class = 12
    skills = ""
    senses = "Blindsight 10 ft., Passive Perception 10"
    languages = ""
    strength = Ability(15)
    dexterity = Ability(14)
    constitution = Ability(12)
    intelligence = Ability(1)
    wisdom = Ability(10)
    charisma = Ability(3)
    speed = 30
    swim_speed = 30
    fly_speed = 0
    climb_speed = 0
    hp_max = 13
    hit_dice = "2d10"
    spells = []


class CopperDragonWyrmling(Monster):
    """
    Bite.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one target. Hit: 7 (1d10
      + 2) piercing damage.
    Breath Weapons.
      The dragon uses one of the following breath weapons.
      
      Acid Breath. The dragon exhales acid in an 20-foot line that is 5 feet
      wide. Each creature in that line must make a DC 11 Dexterity saving
      throw, taking 18 (4d8) acid damage on a failed save, or half as much
      damage on a successful one.
      
      Slowing Breath. The dragon exhales gas in a 1 5-foot cone. Each
      creature in that area must succeed on a DC 11 Constitution saving
      throw. On a failed save, the creature can't use reactions, its speed
      is halved, and it can't make more than one attack on its turn. In
      addition, the creature can use either an action or a bonus action on
      its turn, but not both. These effects last for 1 minute. The creature
      can repeat the saving throw at the end of each of its turns, ending
      the effect on itself with a successful save.
    """
    name = "Copper Dragon Wyrmling"
    description = "Medium dragon, chaotic good"
    challenge_rating = 1
    armor_class = 16
    skills = "Perception +4, Stealth +3"
    senses = "Blindsight 10 ft., Darkvision 60 ft., Passive Perception 14"
    languages = "Draconic"
    strength = Ability(15)
    dexterity = Ability(12)
    constitution = Ability(13)
    intelligence = Ability(14)
    wisdom = Ability(11)
    charisma = Ability(13)
    speed = 30
    swim_speed = 0
    fly_speed = 60
    climb_speed = 30
    hp_max = 22
    hit_dice = "4d8"
    spells = []


class Couatl(Monster):
    """
    Innate Spellcasting.
      The couatl's spellcasting ability is Charisma (spell save DC 14). It
      can innately cast the following spells, requiring only verbal
      components:
      
  
      
      At will: detect evil and good, detect magic, detect thoughts
      
      3/day each: bless, create food and water, cure wounds, lesser
      restoration, protection from poison, sanctuary, shield
      
      1/day each: dream, greater restoration, scrying
    Magic Weapons.
      The couatl's weapon attacks are magical.
    Shielded Mind.
      The couatl is immune to scrying and to any effect that would sense its
      emotions, read its thoughts, or detect its location.
    Bite.
      Melee Weapon Attack: +8 to hit, reach 5 ft., one creature. Hit: 8 (1d6
      + 5) piercing damage, and the target must succeed on a DC 13
      Constitution saving throw or be poisoned for 24 hours. Until this
      poison ends, the target is unconscious. Another creature can use an
      action to shake the target awake.
    Constrict.
      Melee Weapon Attack: +6 to hit, reach 10 ft., one Medium or smaller
      creature. Hit: 10 (2d6 + 3) bludgeoning damage, and the target is
      grappled (escape DC 15). Until this grapple ends, the target is
      restrained, and the couatl can't constrict another target.
    Change Shape.
      The couatl magically polymorphs into a humanoid or beast that has a
      challenge rating equal to or less than its own, or back into its true
      form. It reverts to its true form if it dies. Any equipment it is
      wearing or carrying is absorbed or borne by the new form (the couatl's
      choice).
      
      In a new form, the couatl retains its game statistics and ability to
      speak, but its AC, movement modes, Strength, Dexterity, and other
      actions are replaced by those of the new form, and it gains any
      statistics and capabilities (except class features, legendary actions,
      and lair actions) that the new form has but that it lacks. If the new
      form has a bite attack, the couatl can use its bite in that form.
    """
    name = "Couatl"
    description = "Medium celestial, lawful good"
    challenge_rating = 4
    armor_class = 19
    skills = ""
    senses = "Truesight 120 ft., Passive Perception 15"
    languages = "all, telepathy 120 ft."
    strength = Ability(16)
    dexterity = Ability(20)
    constitution = Ability(17)
    intelligence = Ability(18)
    wisdom = Ability(20)
    charisma = Ability(18)
    speed = 30
    swim_speed = 0
    fly_speed = 90
    climb_speed = 0
    hp_max = 97
    hit_dice = "13d8"
    spells = []


class Crab(Monster):
    """
    Amphibious.
      The crab can breathe air and water.
    Claw.
      Melee Weapon Attack: +0 to hit, reach 5 ft., one target. Hit: 1
      bludgeoning damage.
    """
    name = "Crab"
    description = "Tiny beast, unaligned"
    challenge_rating = 0
    armor_class = 11
    skills = "Stealth +2"
    senses = "Blindsight 30 ft., Passive Perception 9"
    languages = ""
    strength = Ability(2)
    dexterity = Ability(11)
    constitution = Ability(10)
    intelligence = Ability(1)
    wisdom = Ability(8)
    charisma = Ability(2)
    speed = 20
    swim_speed = 20
    fly_speed = 0
    climb_speed = 0
    hp_max = 2
    hit_dice = "1d4"
    spells = []


class Crocodile(Monster):
    """
    Hold Breath.
      The crocodile can hold its breath for 15 minutes.
    Bite.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one creature. Hit: 7
      (1d10 + 2) piercing damage, and the target is grappled (escape DC 12).
      Until this grapple ends, the target is restrained, and the crocodile
      can't bite another target
    """
    name = "Crocodile"
    description = "Large beast, unaligned"
    challenge_rating = 0.5
    armor_class = 12
    skills = "Stealth +2"
    senses = "Passive Perception 10"
    languages = ""
    strength = Ability(15)
    dexterity = Ability(10)
    constitution = Ability(13)
    intelligence = Ability(2)
    wisdom = Ability(10)
    charisma = Ability(5)
    speed = 20
    swim_speed = 20
    fly_speed = 0
    climb_speed = 0
    hp_max = 19
    hit_dice = "3d10"
    spells = []


class CultFanatic(Monster):
    """
    Dark Devotion.
      The fanatic has advantage on saving throws against being charmed or
      frightened.
    Spellcasting.
      The fanatic is a 4th-level spellcaster. Its spell casting ability is
      Wisdom (spell save DC 11, +3 to hit with spell attacks). The fanatic
      has the following cleric spells prepared:
      
  
      
      Cantrips (at will): light, sacred flame, thaumaturgy
      
      - 1st level (4 slots): command, inflict wounds, shield of faith
      
      - 2nd level (3 slots): hold person, spiritual weapon
    Multiattack.
      The fanatic makes two melee attacks.
    Dagger.
      Melee or Ranged Weapon Attack: +4 to hit, reach 5 ft. or range 20/60
      ft., one creature. Hit: 4 (1d4 + 2) piercing damage.
    """
    name = "Cult Fanatic"
    description = "Medium humanoid, any non-good alignment"
    challenge_rating = 2
    armor_class = 13
    skills = "Deception +4, Persuasion +4, Religion +2"
    senses = "Passive Perception 11"
    languages = "any one language (usually Common)"
    strength = Ability(11)
    dexterity = Ability(14)
    constitution = Ability(12)
    intelligence = Ability(10)
    wisdom = Ability(13)
    charisma = Ability(14)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 22
    hit_dice = "6d8"
    spells = ["light", "sacred flame", "thaumaturgy", "command", "inflict wounds", "shield of faith", "hold person", "spiritual weapon"]


class Cultist(Monster):
    """
    Dark Devotion.
      The cultist has advantage on saving throws against being charmed or
      frightened.
    Scimitar.
      Melee Weapon Attack: +3 to hit, reach 5 ft., one creature. Hit: 4 (1d6
      + 1) slashing damage.
    """
    name = "Cultist"
    description = "Medium humanoid, any non-good alignment"
    challenge_rating = 0.125
    armor_class = 12
    skills = "Deception +2, Religion +2"
    senses = "Passive Perception 10"
    languages = "any one language (usually Common)"
    strength = Ability(11)
    dexterity = Ability(12)
    constitution = Ability(10)
    intelligence = Ability(10)
    wisdom = Ability(11)
    charisma = Ability(10)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 9
    hit_dice = "2d8"
    spells = []



from dungeonsheets import spells


def all_spells():
    """Generate all the valid spell classes so far defined."""
    for spell_name, spell in spells.__dict__.items():
        # Check if it's (a) a class and (b) a subclass of ``Spell``
        if isinstance(spell, type) and issubclass(spell, Spell):
            yield spell


def create_spell(**params):
    """Create a new subclass of ``Spell`` with given default parameters.

    Useful for spells that haven't been entered into the ``spells.py``
    file yet.

    Parameters
    ----------
    params : optional
      Saved as attributes of the new class.

    Returns
    -------
    NewSpell
      New spell class, subclass of ``Spell``, with given params.
    """
    NewSpell = Spell
    NewSpell.name = params.get("name", "Unknown Spell")
    NewSpell.level = params.get("level", 9)
    return NewSpell


class Spell:
    """A magical spell castable by a player character."""

    level = 0
    name = "Unknown spell"
    casting_time = "1 action"
    casting_range = "60 ft"
    components = ()
    materials = ""
    duration = "instantaneous"
    ritual = False
    _concentration = False
    magic_school = ""
    classes = ()

    def __str__(self):
        s = self.name
        requirements = list(self.components)
        # Indicate if this is a ritual or a concentration
        indicators = [
            ("R", self.ritual),
            ("C", self.concentration),
            ("$", self.special_material),
        ]
        requirements.extend([letter for letter, is_active in indicators if is_active])
        if len(requirements):
            s += f' ({"/".join(requirements)})'
        return s

    def __repr__(self):
        return f"{self.level} {self.name}"

    def __eq__(self, other):
        return (self.name == other.name) and (self.level == other.level)

    def __hash__(self):
        return 0

    @property
    def component_string(self):
        s = f'{", ".join(self.components)}'
        if "M" in self.components:
            s += f" ({self.materials})"
        return s

    @property
    def concentration(self):
        return ("concentration" in self.duration.lower()) or self._concentration

    @concentration.setter
    def concentration(self, val: bool):
        self._concentration = val

    @property
    def special_material(self):
        return "worth" in self.materials.lower()



from collections import defaultdict

from dungeonsheets import features, weapons
from dungeonsheets.classes.classes import CharClass, SubClass


# PHB
class DraconicBloodline(SubClass):
    """Your innate magic comes from draconic magic that was mingled with your
    blood or that of your ancestors. Most often, sorcerers with this origin
    trace their descent back to a mighty sorcerer of ancient times who made a
    bargain with a dragon or who might even have claimed a dragon parent. Some
    of these bloodlines are well established in the world, but most are
    obscure. Any given sorcerer could be the first of a new bloodline, as a
    result of a pact or some other exceptional circumstance.

    """

    name = "Draconic Bloodline"
    languages = ("draconic",)
    features_by_level = defaultdict(list)
    features_by_level[1] = [features.DragonAncestor, features.DraconicResilience]
    features_by_level[6] = [features.ElementalAffinity]
    features_by_level[14] = [features.DragonWings]
    features_by_level[18] = [features.DraconicPresence]


class WildMagic(SubClass):
    """Your innate magic comes from the wild forces of chaos that underlie the
    order of creation. You might have endured exposure to some form o f raw
    magic, perhaps through a planar portal leading to Limbo, the Elemental
    Planes, or the mysterious Far Realm. Perhaps you were blessed by a powerful
    fey creature or marked by a demon. Or your magic could be a fluke of your
    birth, with no apparent cause or reason. However it came to be, this
    chaotic magic churns within you, waiting for any outlet.

    """

    name = "Wild Magic"
    features_by_level = defaultdict(list)
    features_by_level[1] = [features.WildMagicSurge, features.TidesOfChaos]
    features_by_level[6] = [features.BendLuck]
    features_by_level[14] = [features.ControlledChaos]
    features_by_level[18] = [features.SpellBombardment]


# XGTE
class DivineSoul(SubClass):
    """Sometimes the spark of magic that fuels a sorcerer comes from a divine
    source that glimmers within the soul. Having such a blessed soul is a sign
    that your innate magic might come from a distant but powerful familial
    connection to a divine being. Perhaps your ances- tor was an angel,
    transformed into a mortal and sent to fight in a god's name. Or your birth
    might align with an ancient prophecy, marking you as a servant of the gods
    or a chosen vessel of divine magic.

    A Divine Soul, with a natural magnetism, is seen as a threat by some
    religious hierarchies. As an outsider who commands sacred power, a Divine
    Soul can undermine an existing order by claiming a direct tie to the
    divine.

    In some cultures, only those who can claim the power of a Divine Soul may
    command religious power. In these lands, ecclesiastical positions are
    dominated by a few bloodlines and preserved over generations

    """

    name = "Divine Soul"
    features_by_level = defaultdict(list)
    features_by_level[1] = [features.DivineMagic, features.FavoredByTheGods]
    features_by_level[6] = [features.EmpoweredHealing]
    features_by_level[14] = [features.OtherworldlyWings]
    features_by_level[18] = [features.UnearthlyRecovery]


class ShadowMagic(SubClass):
    """You are a creature of shadow, for your innate magic comes from the
    Shadowfell itself. You might trace your lineage to an entity from that
    place, or perhaps you were exposed to its fell energy and transformed by
    it.

    The power of shadow magic casts a strange pall over your physical
    presence. The spark of life that sustains you is muffled, as if it
    struggles to remain viable against the dark energy that imbues your
    soul. At your option, you can pick from or roll on the Shadow Sorcerer
    Quirks table to create a quirk for your character

    """

    name = "Shadow Magic"
    features_by_level = defaultdict(list)
    features_by_level[1] = [
        features.EyesOfTheDark,
        features.SuperiorDarkvision,
        features.StrengthOfTheGrave,
    ]
    features_by_level[6] = [features.HoundOfIllOmen]
    features_by_level[14] = [features.ShadowWalk]
    features_by_level[18] = [features.UmbralForm]


class StormSorcery(SubClass):
    """Your innate magic comes from the power of elemental air. Many with this
    power can trace their magic back to a near-death experience caused by the
    Great Rain, but perhaps you were born during a howling gale so powerful
    that folk still tell stories of it, or your lineage might include the
    influence of potent air creatures such as djinn. Whatever the case, the
    magic of the storm permeates your being.

    Storm sorcerers are invaluable members of a ship's crew. Their magic allows
    them to exert control over wind and weather in their immediate area. Their
    abilities also prove useful in repelling attacks by sahuagin, pirates,
    and other waterborne threats.

    """

    name = "Storm Sorcery"
    languages = ("primordial",)
    features_by_level = defaultdict(list)
    features_by_level[1] = [features.TempestuousMagic]
    features_by_level[6] = [features.HeartOfTheStorm, features.StormGuide]
    features_by_level[14] = [features.StormsFury]
    features_by_level[18] = [features.WindSoul]


class Sorceror(CharClass):
    name = "Sorceror"
    hit_dice_faces = 6
    subclass_select_level = 1
    saving_throw_proficiencies = ("constitution", "charisma")
    primary_abilities = ("charisma",)
    _proficiencies_text = (
        "daggers",
        "darts",
        "slings",
        "quarterstaffs",
        "light crossbows",
    )
    weapon_proficiencies = (
        weapons.Dagger,
        weapons.Dart,
        weapons.Sling,
        weapons.Quarterstaff,
        weapons.LightCrossbow,
    )
    multiclass_weapon_proficiencies = ()
    _multiclass_proficiencies_text = ()
    class_skill_choices = (
        "Arcana",
        "Deception",
        "Insight",
        "Intimidation",
        "Persuasion",
        "Religion",
    )
    features_by_level = defaultdict(list)
    features_by_level[2] = [features.FontOfMagic]
    features_by_level[3] = [features.Metamagic]
    features_by_level[20] = [features.SorcerousRestoration]
    subclasses_available = (
        DraconicBloodline,
        WildMagic,
        DivineSoul,
        ShadowMagic,
        StormSorcery,
    )
    spellcasting_ability = "charisma"
    spell_slots_by_level = {
        # char_lvl: (cantrips, 1st, 2nd, 3rd, ...)
        1: (4, 2, 0, 0, 0, 0, 0, 0, 0, 0),
        2: (4, 3, 0, 0, 0, 0, 0, 0, 0, 0),
        3: (4, 4, 2, 0, 0, 0, 0, 0, 0, 0),
        4: (5, 4, 3, 0, 0, 0, 0, 0, 0, 0),
        5: (5, 4, 3, 2, 0, 0, 0, 0, 0, 0),
        6: (5, 4, 3, 3, 0, 0, 0, 0, 0, 0),
        7: (5, 4, 3, 3, 1, 0, 0, 0, 0, 0),
        8: (5, 4, 3, 3, 2, 0, 0, 0, 0, 0),
        9: (5, 4, 3, 3, 3, 1, 0, 0, 0, 0),
        10: (6, 4, 3, 3, 3, 2, 0, 0, 0, 0),
        11: (6, 4, 3, 3, 3, 2, 1, 0, 0, 0),
        12: (6, 4, 3, 3, 3, 2, 1, 0, 0, 0),
        13: (6, 4, 3, 3, 3, 2, 1, 1, 0, 0),
        14: (6, 4, 3, 3, 3, 2, 1, 1, 0, 0),
        15: (6, 4, 3, 3, 3, 2, 1, 1, 1, 0),
        16: (6, 4, 3, 3, 3, 2, 1, 1, 1, 0),
        17: (6, 4, 3, 3, 3, 2, 1, 1, 1, 1),
        18: (6, 4, 3, 3, 3, 3, 1, 1, 1, 1),
        19: (6, 4, 3, 3, 3, 3, 2, 1, 1, 1),
        20: (6, 4, 3, 3, 3, 3, 2, 2, 1, 1),
    }
